@@ -1,6 +1,6 @@
 
 /*
- * $Id: acl.c,v 1.185.2.11 1998/12/04 17:14:45 wessels Exp $
+ * $Id: acl.c,v 1.197.2.12 1999/09/22 05:57:55 wessels Exp $
  *
  * DEBUG: section 28    Access Control
  * AUTHOR: Duane Wessels
@@ -47,13 +47,13 @@ static void aclParseWordList(void *curlist);
 static void aclParseProtoList(void *curlist);
 static void aclParseMethodList(void *curlist);
 static void aclParseTimeSpec(void *curlist);
-static void aclParseSnmpComm(void *curlist);
 static void aclParseIntRange(void *curlist);
 static char *strtokFile(void);
 static void aclDestroyAclList(acl_list * list);
 static void aclDestroyTimeList(acl_time_data * data);
 static void aclDestroyIntRange(intrange *);
 static FREE aclFreeProxyAuthUser;
+static struct _acl *aclFindByName(const char *name);
 static int aclMatchAcl(struct _acl *, aclCheck_t *);
 static int aclMatchIntegerRange(intrange * data, int i);
 static int aclMatchTime(acl_time_data * data, time_t when);
@@ -61,10 +61,16 @@ static int aclMatchUser(wordlist * data, const char *ident);
 static int aclMatchIp(void *dataptr, struct in_addr c);
 static int aclMatchDomainList(void *dataptr, const char *);
 static int aclMatchIntegerRange(intrange * data, int i);
+#if SQUID_SNMP
+static int aclMatchWordList(wordlist *, const char *);
+#endif
 static squid_acl aclStrToType(const char *s);
 static int decode_addr(const char *, struct in_addr *, struct in_addr *);
 static void aclCheck(aclCheck_t * checklist);
 static void aclCheckCallback(aclCheck_t * checklist, allow_t answer);
+#if USE_IDENT
+static IDCB aclLookupIdentDone;
+#endif
 static IPH aclLookupDstIPDone;
 static IPH aclLookupDstIPforASNDone;
 static FQDNH aclLookupSrcFQDNDone;
@@ -147,6 +153,8 @@ aclStrToType(const char *s)
 	return ACL_SRC_IP;
     if (!strcmp(s, "dst"))
 	return ACL_DST_IP;
+    if (!strcmp(s, "myip"))
+	return ACL_MY_IP;
     if (!strcmp(s, "domain"))
 	return ACL_DST_DOMAIN;
     if (!strcmp(s, "dstdomain"))
@@ -167,8 +175,10 @@ aclStrToType(const char *s)
 	return ACL_URL_REGEX;
     if (!strcmp(s, "port"))
 	return ACL_URL_PORT;
+#if USE_IDENT
     if (!strcmp(s, "ident"))
 	return ACL_IDENT;
+#endif
     if (!strncmp(s, "proto", 5))
 	return ACL_PROTO;
     if (!strcmp(s, "method"))
@@ -181,8 +191,10 @@ aclStrToType(const char *s)
 	return ACL_SRC_ASN;
     if (!strcmp(s, "dst_as"))
 	return ACL_DST_ASN;
+#if SQUID_SNMP
     if (!strcmp(s, "snmp_community"))
-	return ACL_SNMP_COMM;
+	return ACL_SNMP_COMMUNITY;
+#endif
     if (!strcmp(s, "src_rtt"))
 	return ACL_NETDB_SRC_RTT;
 #if USE_ARP_ACL
@@ -199,6 +211,8 @@ aclTypeToStr(squid_acl type)
 	return "src";
     if (type == ACL_DST_IP)
 	return "dst";
+    if (type == ACL_MY_IP)
+	return "myip";
     if (type == ACL_DST_DOMAIN)
 	return "dstdomain";
     if (type == ACL_SRC_DOMAIN)
@@ -215,8 +229,10 @@ aclTypeToStr(squid_acl type)
 	return "url_regex";
     if (type == ACL_URL_PORT)
 	return "port";
+#if USE_IDENT
     if (type == ACL_IDENT)
 	return "ident";
+#endif
     if (type == ACL_PROTO)
 	return "proto";
     if (type == ACL_METHOD)
@@ -229,8 +245,10 @@ aclTypeToStr(squid_acl type)
 	return "src_as";
     if (type == ACL_DST_ASN)
 	return "dst_as";
-    if (type == ACL_SNMP_COMM)
+#if SQUID_SNMP
+    if (type == ACL_SNMP_COMMUNITY)
 	return "snmp_community";
+#endif
     if (type == ACL_NETDB_SRC_RTT)
 	return "src_rtt";
 #if USE_ARP_ACL
@@ -240,7 +258,7 @@ aclTypeToStr(squid_acl type)
     return "ERROR";
 }
 
-acl *
+static acl *
 aclFindByName(const char *name)
 {
     acl *a;
@@ -609,23 +627,6 @@ aclParseDomainList(void *curlist)
     }
 }
 
-static void
-aclParseSnmpComm(void *data)
-{
-    acl_snmp_comm **q = data;
-    acl_snmp_comm *p;
-    char *t;
-    t = strtok(NULL, w_space);
-    if (t) {
-	p = xcalloc(1, sizeof(acl_snmp_comm));
-	p->name = xstrdup(t);
-	p->community = NULL;
-	*q = p;
-    }
-    t = strtok(NULL, w_space);
-    return;
-}
-
 void
 aclParseAclLine(acl ** head)
 {
@@ -672,9 +673,15 @@ aclParseAclLine(acl ** head)
 	debug(28, 3) ("aclParseAclLine: Appending to '%s'\n", aclname);
 	new_acl = 0;
     }
+    /*
+     * Here we set AclMatchedName in case we need to use it in a
+     * warning message in aclDomainCompare().
+     */
+    AclMatchedName = aclname;	/* ugly */
     switch (A->type) {
     case ACL_SRC_IP:
     case ACL_DST_IP:
+    case ACL_MY_IP:
 	aclParseIpList(&A->data);
 	break;
     case ACL_SRC_DOMAIN:
@@ -686,6 +693,7 @@ aclParseAclLine(acl ** head)
 	break;
     case ACL_URL_REGEX:
     case ACL_URLPATH_REGEX:
+    case ACL_BROWSER:
     case ACL_SRC_DOM_REGEX:
     case ACL_DST_DOM_REGEX:
 	aclParseRegexList(&A->data);
@@ -698,19 +706,17 @@ aclParseAclLine(acl ** head)
     case ACL_URL_PORT:
 	aclParseIntRange(&A->data);
 	break;
+#if USE_IDENT
     case ACL_IDENT:
-	Config.onoff.ident_lookup = 1;
 	aclParseWordList(&A->data);
 	break;
+#endif
     case ACL_PROTO:
 	aclParseProtoList(&A->data);
 	break;
     case ACL_METHOD:
 	aclParseMethodList(&A->data);
 	break;
-    case ACL_BROWSER:
-	aclParseRegexList(&A->data);
-	break;
     case ACL_PROXY_AUTH:
 	aclParseWordList(&A->data);
 	if (!proxy_auth_cache) {
@@ -719,9 +725,11 @@ aclParseAclLine(acl ** head)
 	    assert(proxy_auth_cache);
 	}
 	break;
-    case ACL_SNMP_COMM:
-	aclParseSnmpComm(&A->data);
+#if SQUID_SNMP
+    case ACL_SNMP_COMMUNITY:
+	aclParseWordList(&A->data);
 	break;
+#endif
 #if USE_ARP_ACL
     case ACL_SRC_ARP:
 	aclParseArpList(&A->data);
@@ -732,6 +740,10 @@ aclParseAclLine(acl ** head)
 	fatal("Bad ACL type");
 	break;
     }
+    /*
+     * Clear AclMatchedName from our temporary hack
+     */
+    AclMatchedName = NULL;	/* ugly */
     if (!new_acl)
 	return;
     if (A->data == NULL) {
@@ -770,6 +782,17 @@ aclGetDenyInfoPage(acl_deny_info_list ** head, const char *name)
     return -1;
 }
 
+/* does name lookup, returns if it is a proxy_auth acl */
+int
+aclIsProxyAuth(const char *name)
+{
+    acl *a = aclFindByName(name);
+    if (a)
+	return a->type == ACL_PROXY_AUTH;
+    return 0;
+}
+
+
 /* maex@space.net (05.09.96)
  *    get the info for redirecting "access denied" to info pages
  *      TODO (probably ;-)
@@ -887,7 +910,7 @@ aclParseAccessLine(acl_access ** head)
     for (B = *head, T = head; B; T = &B->next, B = B->next);
     *T = A;
     /* We lock _acl_access structures in aclCheck() */
-    cbdataAdd(A, MEM_ACL_ACCESS);
+    cbdataAdd(A, memFree, MEM_ACL_ACCESS);
 }
 
 /**************/
@@ -956,7 +979,7 @@ aclMatchUser(wordlist * data, const char *user)
     debug(28, 3) ("aclMatchUser: checking '%s'\n", user);
     while (data) {
 	debug(28, 3) ("aclMatchUser: looking for '%s'\n", data->key);
-	if (strcmp(data->key, "REQUIRED") == 0 && *user != '\0')
+	if (strcmp(data->key, "REQUIRED") == 0 && *user != '\0' && strcmp(user, "-") != 0)
 	    return 1;
 	if (strcmp(data->key, user) == 0)
 	    return 1;
@@ -980,7 +1003,7 @@ aclDecodeProxyAuth(const char *proxy_auth, char **user, char **password, char *b
     /* Trim trailing \n before decoding */
     strtok(sent_auth, "\n");
     /* Trim leading whitespace before decoding */
-    while (isspace(*proxy_auth))
+    while (xisspace(*proxy_auth))
 	proxy_auth++;
     cleartext = uudecode(sent_auth);
     xfree(sent_auth);
@@ -998,14 +1021,15 @@ aclDecodeProxyAuth(const char *proxy_auth, char **user, char **password, char *b
 }
 
 /* aclMatchProxyAuth can return three exit codes:
- * 0 : No such user; invalid Proxy-authorization: header;
- * ask for Proxy-Authorization: header
+ * 0 : user denied access
  * 1 : user validated OK
  * -1 : check the password for this user via an external authenticator
+ * -2 : invalid Proxy-authorization: header;
+ * ask for Proxy-Authorization: header
  */
 
 static int
-aclMatchProxyAuth(const char *proxy_auth, acl_proxy_auth_user * auth_user, aclCheck_t * checklist)
+aclMatchProxyAuth(wordlist * data, const char *proxy_auth, acl_proxy_auth_user * auth_user, aclCheck_t * checklist)
 {
     /* checklist is used to register user name when identified, nothing else */
     LOCAL_ARRAY(char, login_buf, USER_IDENT_SZ);
@@ -1013,56 +1037,66 @@ aclMatchProxyAuth(const char *proxy_auth, acl_proxy_auth_user * auth_user, aclCh
 
     if (!aclDecodeProxyAuth(proxy_auth, &user, &password, login_buf, sizeof(login_buf)))
 	/* No or invalid Proxy-Auth header */
-	return 0;
+	return -2;
 
     debug(28, 5) ("aclMatchProxyAuth: checking user '%s'\n", user);
 
-    if (!auth_user) {
-	/* see if we already know this user */
-	auth_user = hash_lookup(proxy_auth_cache, user);
-	if (!auth_user) {
-	    /* user not yet known, ask external authenticator */
-	    debug(28, 4) ("aclMatchProxyAuth: user '%s' not yet known\n", user);
-	    return -1;
-	} else if ((0 == strcmp(auth_user->passwd, password)) &&
-	    (auth_user->expiretime > current_time.tv_sec)) {
-	    /* user already known and valid */
-	    debug(28, 5) ("aclMatchProxyAuth: user '%s' previously validated\n",
+    if (auth_user) {
+	/* This should be optimized to a boolean argument indicating that the
+	 * password is invalid, instead of passing full acl_proxy_auth_user
+	 * structures, and all messing with checklist->proxy_auth should
+	 * be restricted the functions that deal with the authenticator.
+	 */
+	assert(auth_user == checklist->auth_user);
+	checklist->auth_user = NULL;	/* get rid of that special reference */
+	/* Check result from external validation */
+	if (auth_user->passwd_ok != 1) {
+	    /* password was checked but did not match */
+	    assert(auth_user->passwd_ok == 0);
+	    debug(28, 4) ("aclMatchProxyAuth: authentication failed for user '%s'\n",
 		user);
-	    /* copy username to request for logging on client-side */
-	    xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
-	    return 1;
-	} else {
-	    /* password mismatch/timeout */
-	    debug(28, 4) ("aclMatchProxyAuth: user '%s' password mismatch/timeout\n",
-		user);
-	    /* remove this user from the hash, making him unknown */
-	    hash_remove_link(proxy_auth_cache, (hash_link *) auth_user);
 	    aclFreeProxyAuthUser(auth_user);
 	    /* copy username to request for logging on client-side unless ident
 	     * is known (do not override ident with false proxy auth names) */
 	    if (!*checklist->request->user_ident)
 		xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
-	    return -1;
+	    return -2;
+	} else {
+	    /* password was checked and did match */
+	    debug(28, 4) ("aclMatchProxyAuth: user '%s' validated OK\n", user);
+	    /* store validated user in hash, after filling in expiretime */
+	    xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
+	    auth_user->expiretime = current_time.tv_sec + Config.authenticateTTL;
+	    hash_join(proxy_auth_cache, (hash_link *) auth_user);
+	    /* Continue checking below, as normal */
 	}
-	/* NOTREACHED */
-    } else {
-	/* Check result from external validation */
-	if (checklist->auth_user->passwd_ok != 1) {
-	    /* password was checked but did not match */
-	    assert(checklist->auth_user->passwd_ok == 0);
-	    debug(28, 4) ("aclMatchProxyAuth: authentication failed for user '%s'\n",
-		user);
-	    return 0;
-	}
-	debug(28, 4) ("aclMatchProxyAuth: user '%s' validated OK\n", user);
-	/* store validated user in hash, after filling in expiretime */
-	checklist->auth_user->expiretime = current_time.tv_sec + Config.authenticateTTL;
-	hash_join(proxy_auth_cache, (hash_link *) checklist->auth_user);
-
-	return 1;
     }
+    /* see if we already know this user */
+    auth_user = hash_lookup(proxy_auth_cache, user);
 
+    if (!auth_user) {
+	/* user not yet known, ask external authenticator */
+	debug(28, 4) ("aclMatchProxyAuth: user '%s' not yet known\n", user);
+	return -1;
+    } else if ((0 == strcmp(auth_user->passwd, password)) &&
+	(auth_user->expiretime > current_time.tv_sec)) {
+	/* user already known and valid */
+	debug(28, 5) ("aclMatchProxyAuth: user '%s' previously validated\n",
+	    user);
+	/* copy username to request for logging on client-side */
+	xstrncpy(checklist->request->user_ident, user, USER_IDENT_SZ);
+	return aclMatchUser(data, user);
+    } else {
+	/* password mismatch/timeout */
+	debug(28, 4) ("aclMatchProxyAuth: user '%s' password mismatch/timeout\n",
+	    user);
+	/* remove this user from the hash, making him unknown */
+	hash_remove_link(proxy_auth_cache, (hash_link *) auth_user);
+	aclFreeProxyAuthUser(auth_user);
+	/* ask the external authenticator in case the password is changed */
+	/* wrong password will be trapped above so this does not loop */
+	return -1;
+    }
     /* NOTREACHED */
 
 }
@@ -1175,6 +1209,21 @@ aclMatchTime(acl_time_data * data, time_t when)
     return data->weekbits & (1 << tm.tm_wday) ? 1 : 0;
 }
 
+#if SQUID_SNMP
+static int
+aclMatchWordList(wordlist * w, const char *word)
+{
+    debug(28, 3) ("aclMatchWordList: looking for '%s'\n", word);
+    while (w != NULL) {
+	debug(28, 3) ("aclMatchWordList: checking '%s'\n", w->key);
+	if (!strcmp(w->key, word))
+	    return 1;
+	w = w->next;
+    }
+    return 0;
+}
+#endif
+
 static int
 aclMatchAcl(acl * ae, aclCheck_t * checklist)
 {
@@ -1182,6 +1231,7 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
     const ipcache_addrs *ia = NULL;
     const char *fqdn = NULL;
     char *esc_buf;
+    const char *header;
     int k;
     if (!ae)
 	return 0;
@@ -1190,6 +1240,9 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
     case ACL_SRC_IP:
 	return aclMatchIp(&ae->data, checklist->src_addr);
 	/* NOTREACHED */
+    case ACL_MY_IP:
+	return aclMatchIp(&ae->data, checklist->my_addr);
+	/* NOTREACHED */
     case ACL_DST_IP:
 	ia = ipcache_gethostbyname(r->host, IP_LOOKUP_IF_MISS);
 	if (ia) {
@@ -1279,9 +1332,16 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
     case ACL_URL_PORT:
 	return aclMatchIntegerRange(ae->data, r->port);
 	/* NOTREACHED */
+#if USE_IDENT
     case ACL_IDENT:
-	return aclMatchUser(ae->data, checklist->ident);
+	if (checklist->ident[0]) {
+	    return aclMatchUser(ae->data, checklist->ident);
+	} else {
+	    checklist->state[ACL_IDENT] = ACL_LOOKUP_NEEDED;
+	    return 0;
+	}
 	/* NOTREACHED */
+#endif
     case ACL_PROTO:
 	return aclMatchInteger(ae->data, r->protocol);
 	/* NOTREACHED */
@@ -1292,31 +1352,43 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	return aclMatchRegex(ae->data, checklist->browser);
 	/* NOTREACHED */
     case ACL_PROXY_AUTH:
-	if (!r->flags.accelerated) {
+	if (NULL == r) {
+	    return -1;
+	} else if (!r->flags.accelerated) {
 	    /* Proxy authorization on proxy requests */
-	    k = aclMatchProxyAuth(httpHeaderGetStr(&checklist->request->header,
-		    HDR_PROXY_AUTHORIZATION),
-		checklist->auth_user,
-		checklist);
+	    header = httpHeaderGetStr(&checklist->request->header,
+		HDR_PROXY_AUTHORIZATION);
 	} else if (r->flags.internal) {
 	    /* WWW authorization on accelerated internal requests */
-	    k = aclMatchProxyAuth(httpHeaderGetStr(&checklist->request->header,
-		    HDR_AUTHORIZATION),
-		checklist->auth_user,
-		checklist);
+	    header = httpHeaderGetStr(&checklist->request->header,
+		HDR_AUTHORIZATION);
 	} else {
 #if AUTH_ON_ACCELERATION
 	    /* WWW authorization on accelerated requests */
-	    k = aclMatchProxyAuth(httpHeaderGetStr(&checklist->request->header,
-		    HDR_AUTHORIZATION),
-		checklist->auth_user,
-		checklist);
+	    header = httpHeaderGetStr(&checklist->request->header,
+		HDR_AUTHORIZATION);
 #else
 	    debug(28, 1) ("aclMatchAcl: proxy_auth %s not applicable on accelerated requests.\n", ae->name);
 	    return -1;
 #endif
 	}
-	if (k == 0) {
+	/*
+	 * Register that we used the proxy authentication header so that
+	 * it is not forwarded to the next proxy
+	 */
+	r->flags.used_proxy_auth = 1;
+	/* Check the password */
+	switch (aclMatchProxyAuth(ae->data,
+		header,
+		checklist->auth_user,
+		checklist)) {
+	case 0:
+	    /* Correct password, but was not allowed in this ACL */
+	    return 0;
+	case 1:
+	    /* user validated OK */
+	    return 1;
+	case -2:
 	    /* no such user OR we need a proxy authentication header */
 	    checklist->state[ACL_PROXY_AUTH] = ACL_PROXY_AUTH_NEEDED;
 	    /*
@@ -1324,15 +1396,7 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	     * return codes here
 	     */
 	    return 0;
-	} else if (k == 1) {
-	    /*
-	     * Authentication successful. Register that we used the proxy
-	     * authentication header so that it is not forwarded to the
-	     * next proxy
-	     */
-	    r->flags.used_proxy_auth = 1;
-	    return 1;
-	} else if (k == -1) {
+	case -1:
 	    /*
 	     * we need to validate the password
 	     */
@@ -1340,8 +1404,10 @@ aclMatchAcl(acl * ae, aclCheck_t * checklist)
 	    return 0;
 	}
 	/* NOTREACHED */
-    case ACL_SNMP_COMM:
-	return asnMatchIp(ae->data, checklist->src_addr);
+#if SQUID_SNMP
+    case ACL_SNMP_COMMUNITY:
+	return aclMatchWordList(ae->data, checklist->snmp_community);
+#endif
     case ACL_SRC_ASN:
 	return asnMatchIp(ae->data, checklist->src_addr);
     case ACL_DST_ASN:
@@ -1466,12 +1532,24 @@ aclCheck(aclCheck_t * checklist)
 	     */
 	    allow = ACCESS_REQ_PROXY_AUTH;
 	    match = -1;
-	} else if (checklist->state[ACL_IDENT] == ACL_LOOKUP_NEEDED) {
-	    debug(28, 3) ("aclCheck: Doing ident lookup\n");
-	    /* XXX how to do ident lookup? */
-	    checklist->state[ACL_IDENT] = ACL_LOOKUP_PENDING;
-	    return;
 	}
+#if USE_IDENT
+	else if (checklist->state[ACL_IDENT] == ACL_LOOKUP_NEEDED) {
+	    debug(28, 3) ("aclCheck: Doing ident lookup\n");
+	    if (cbdataValid(checklist->conn)) {
+		identStart(&checklist->conn->me, &checklist->conn->peer,
+		    aclLookupIdentDone, checklist);
+		checklist->state[ACL_IDENT] = ACL_LOOKUP_PENDING;
+		return;
+	    } else {
+		debug(28, 1) ("aclCheck: Can't start ident lookup. No client connection\n");
+		cbdataUnlock(checklist->conn);
+		checklist->conn = NULL;
+		allow = 0;
+		match = -1;
+	    }
+	}
+#endif
 	/*
 	 * We are done with this _acl_access entry.  Either the request
 	 * is allowed, denied, requires authentication, or we move on to
@@ -1506,6 +1584,12 @@ aclChecklistFree(aclCheck_t * checklist)
     if (checklist->request)
 	requestUnlink(checklist->request);
     checklist->request = NULL;
+#if USE_IDENT
+    if (checklist->conn) {
+	cbdataUnlock(checklist->conn);
+	checklist->conn = NULL;
+    }
+#endif
     cbdataFree(checklist);
 }
 
@@ -1521,6 +1605,27 @@ aclCheckCallback(aclCheck_t * checklist, allow_t answer)
     aclChecklistFree(checklist);
 }
 
+#if USE_IDENT
+static void
+aclLookupIdentDone(const char *ident, void *data)
+{
+    aclCheck_t *checklist = data;
+    if (ident) {
+	xstrncpy(checklist->ident, ident, sizeof(checklist->ident));
+	xstrncpy(checklist->request->user_ident, ident, sizeof(checklist->request->user_ident));
+    } else {
+	xstrncpy(checklist->ident, "-", sizeof(checklist->ident));
+    }
+    /*
+     * Cache the ident result in the connection, to avoid redoing ident lookup
+     * over and over on persistent connections
+     */
+    if (cbdataValid(checklist->conn) && !checklist->conn->ident[0])
+	xstrncpy(checklist->conn->ident, checklist->ident, sizeof(checklist->conn->ident));
+    aclCheck(checklist);
+}
+#endif
+
 static void
 aclLookupDstIPDone(const ipcache_addrs * ia, void *data)
 {
@@ -1571,12 +1676,13 @@ aclCheck_t *
 aclChecklistCreate(const acl_access * A,
     request_t * request,
     struct in_addr src_addr,
+    struct in_addr my_addr,
     const char *user_agent,
     const char *ident)
 {
     int i;
     aclCheck_t *checklist = memAllocate(MEM_ACLCHECK_T);
-    cbdataAdd(checklist, MEM_ACLCHECK_T);
+    cbdataAdd(checklist, memFree, MEM_ACLCHECK_T);
     checklist->access_list = A;
     /*
      * aclCheck() makes sure checklist->access_list is a valid
@@ -1586,12 +1692,15 @@ aclChecklistCreate(const acl_access * A,
     if (request != NULL)
 	checklist->request = requestLink(request);
     checklist->src_addr = src_addr;
+    checklist->my_addr = my_addr;
     for (i = 0; i < ACL_ENUM_MAX; i++)
 	checklist->state[i] = ACL_LOOKUP_NONE;
     if (user_agent)
 	xstrncpy(checklist->browser, user_agent, BROWSERNAMELEN);
+#if USE_IDENT
     if (ident)
 	xstrncpy(checklist->ident, ident, USER_IDENT_SZ);
+#endif
     checklist->auth_user = NULL;	/* init to NULL */
     return checklist;
 }
@@ -1621,7 +1730,7 @@ aclDestroyTimeList(acl_time_data * data)
     acl_time_data *next = NULL;
     for (; data; data = next) {
 	next = data->next;
-	memFree(MEM_ACL_TIME_DATA, data);
+	memFree(data, MEM_ACL_TIME_DATA);
     }
 }
 
@@ -1633,7 +1742,7 @@ aclDestroyRegexList(relist * data)
 	next = data->next;
 	regfree(&data->regex);
 	safe_free(data->pattern);
-	memFree(MEM_RELIST, data);
+	memFree(data, MEM_RELIST);
     }
 }
 
@@ -1643,13 +1752,13 @@ aclFreeProxyAuthUser(void *data)
     acl_proxy_auth_user *u = data;
     xfree(u->user);
     xfree(u->passwd);
-    memFree(MEM_ACL_PROXY_AUTH_USER, u);
+    memFree(u, MEM_ACL_PROXY_AUTH_USER);
 }
 
 static void
 aclFreeIpData(void *p)
 {
-    memFree(MEM_ACL_IP_DATA, p);
+    memFree(p, MEM_ACL_IP_DATA);
 }
 
 void
@@ -1663,6 +1772,7 @@ aclDestroyAcls(acl ** head)
 	switch (a->type) {
 	case ACL_SRC_IP:
 	case ACL_DST_IP:
+	case ACL_MY_IP:
 	    splay_destroy(a->data, aclFreeIpData);
 	    break;
 	case ACL_SRC_ARP:
@@ -1670,7 +1780,12 @@ aclDestroyAcls(acl ** head)
 	case ACL_SRC_DOMAIN:
 	    splay_destroy(a->data, xfree);
 	    break;
+#if SQUID_SNMP
+	case ACL_SNMP_COMMUNITY:
+#endif
+#if USE_IDENT
 	case ACL_IDENT:
+#endif
 	case ACL_PROXY_AUTH:
 	    wordlistDestroy((wordlist **) & a->data);
 	    break;
@@ -1680,12 +1795,15 @@ aclDestroyAcls(acl ** head)
 	case ACL_URL_REGEX:
 	case ACL_URLPATH_REGEX:
 	case ACL_BROWSER:
+	case ACL_SRC_DOM_REGEX:
+	case ACL_DST_DOM_REGEX:
 	    aclDestroyRegexList(a->data);
 	    break;
 	case ACL_PROTO:
 	case ACL_METHOD:
 	case ACL_SRC_ASN:
 	case ACL_DST_ASN:
+	case ACL_NETDB_SRC_RTT:
 	    intlistDestroy((intlist **) & a->data);
 	    break;
 	case ACL_URL_PORT:
@@ -1693,11 +1811,11 @@ aclDestroyAcls(acl ** head)
 	    break;
 	case ACL_NONE:
 	default:
-	    assert(0);
+	    debug(28, 1) ("aclDestroyAcls: no case for ACL type %d\n", a->type);
 	    break;
 	}
 	safe_free(a->cfgline);
-	memFree(MEM_ACL, a);
+	memFree(a, MEM_ACL);
     }
     *head = NULL;
 }
@@ -1708,7 +1826,7 @@ aclDestroyAclList(acl_list * list)
     acl_list *next = NULL;
     for (; list; list = next) {
 	next = list->next;
-	memFree(MEM_ACL_LIST, list);
+	memFree(list, MEM_ACL_LIST);
     }
 }
 
@@ -1772,15 +1890,33 @@ aclDomainCompare(const void *data, splayNode * n)
 {
     const char *d1 = data;
     const char *d2 = n->data;
-    int l1 = strlen(d1);
-    int l2 = strlen(d2);
-    while (d1[l1] == d2[l2]) {
+    int l1;
+    int l2;
+    while ('.' == *d1)
+	d1++;
+    while ('.' == *d2)
+	d2++;
+    l1 = strlen(d1);
+    l2 = strlen(d2);
+    while (d1[--l1] == d2[--l2]) {
 	if ((l1 == 0) && (l2 == 0))
 	    return 0;		/* d1 == d2 */
-	if (l1-- == 0)
+	if (0 == l1) {
+	    if ('.' == d2[l2 - 1]) {
+		debug(28, 0) ("WARNING: %s is a subdomain of %s\n", d2, d1);
+		debug(28, 0) ("WARNING: This may break Splay tree searching\n");
+		debug(28, 0) ("WARNING: You should remove '%s' from the ACL named '%s'\n", d2, AclMatchedName);
+	    }
 	    return -1;		/* d1 < d2 */
-	if (l2-- == 0)
+	}
+	if (0 == l2) {
+	    if ('.' == d1[l1 - 1]) {
+		debug(28, 0) ("WARNING: %s is a subdomain of %s\n", d1, d2);
+		debug(28, 0) ("WARNING: This may break Splay tree searching\n");
+		debug(28, 0) ("WARNING: You should remove '%s' from the ACL named '%s'\n", d1, AclMatchedName);
+	    }
 	    return 1;		/* d1 > d2 */
+	}
     }
     return (d1[l1] - d2[l2]);
 }
@@ -1799,20 +1935,18 @@ aclHostDomainCompare(const void *data, splayNode * n)
     l1 = strlen(h);
     l2 = strlen(d);
     /* h != d */
-    while (h[l1] == d[l2]) {
+    while (xtolower(h[--l1]) == xtolower(d[--l2])) {
 	if (l1 == 0)
 	    break;
 	if (l2 == 0)
 	    break;
-	l1--;
-	l2--;
     }
     /* a '.' is a special case */
-    if ((h[l1] == '.') || (l1 == 0))
+    if ((h[l1] == '.') && (l1 == 0))
 	return -1;		/* domain(h) < d */
-    if ((d[l2] == '.') || (l2 == 0))
+    if ((d[l2] == '.') && (l2 == 0))
 	return 1;		/* domain(h) > d */
-    return (h[l1] - d[l2]);
+    return (xtolower(h[l1]) - xtolower(d[l2]));
 }
 
 /* compare two network specs
@@ -1987,20 +2121,30 @@ aclDumpGeneric(const acl * a)
     switch (a->type) {
     case ACL_SRC_IP:
     case ACL_DST_IP:
+    case ACL_MY_IP:
 	return aclDumpIpList(a->data);
 	break;
     case ACL_SRC_DOMAIN:
     case ACL_DST_DOMAIN:
-    case ACL_IDENT:
-    case ACL_PROXY_AUTH:
 	return aclDumpDomainList(a->data);
 	break;
+#if SQUID_SNMP
+    case ACL_SNMP_COMMUNITY:
+#endif
+#if USE_IDENT
+    case ACL_IDENT:
+#endif
+    case ACL_PROXY_AUTH:
+	return wordlistDup(a->data);
+	break;
     case ACL_TIME:
 	return aclDumpTimeSpecList(a->data);
 	break;
     case ACL_URL_REGEX:
     case ACL_URLPATH_REGEX:
     case ACL_BROWSER:
+    case ACL_SRC_DOM_REGEX:
+    case ACL_DST_DOM_REGEX:
 	return aclDumpRegexList(a->data);
 	break;
     case ACL_SRC_ASN:
@@ -2023,6 +2167,7 @@ aclDumpGeneric(const acl * a)
 #endif
     case ACL_NONE:
     default:
+	debug(28, 1) ("aclDumpGeneric: no case for ACL type %d\n", a->type);
 	break;
     }
     return NULL;
@@ -2057,8 +2202,6 @@ aclDumpGeneric(const acl * a)
  *       Original (BSD-specific) code no longer works.
  */
 
-#include "squid.h"
-
 #include <sys/sysctl.h>
 #ifdef _SQUID_LINUX_
 #include <net/if_arp.h>
