@@ -1,6 +1,6 @@
 
 /*
- * $Id: client_side.c,v 1.420.2.14 1998/12/09 20:50:41 wessels Exp $
+ * $Id: client_side.c,v 1.440.2.11 1999/09/23 00:06:53 wessels Exp $
  *
  * DEBUG: section 33    Client-side Routines
  * AUTHOR: Duane Wessels
@@ -41,9 +41,21 @@
 #endif
 #include <netinet/tcp.h>
 #include <net/if.h>
+#if HAVE_IP_COMPAT_H
 #include <ip_compat.h>
+#elif HAVE_NETINET_IP_COMPAT_H
+#include <netinet/ip_compat.h>
+#endif
+#if HAVE_IP_FIL_H
 #include <ip_fil.h>
+#elif HAVE_NETINET_IP_FIL_H
+#include <netinet/ip_fil.h>
+#endif
+#if HAVE_IP_NAT_H
 #include <ip_nat.h>
+#elif HAVE_NETINET_IP_NAT_H
+#include <netinet/ip_nat.h>
+#endif
 #endif
 
 
@@ -53,7 +65,6 @@
 #endif
 
 static const char *const crlf = "\r\n";
-static const char *const proxy_auth_challenge_fmt = "Basic realm=\"%s\"";
 
 #define REQUEST_BUF_SIZE 4096
 #define FAILURE_MODE_TIME 300
@@ -64,10 +75,8 @@ static CWCB clientWriteComplete;
 static PF clientReadRequest;
 static PF connStateFree;
 static PF requestTimeout;
-static int CheckQuickAbort2(const clientHttpRequest *);
 static int clientCheckTransferDone(clientHttpRequest *);
 static int clientGotNotEnough(clientHttpRequest *);
-static void CheckQuickAbort(clientHttpRequest *);
 static void checkFailureRatio(err_type, hier_code);
 static void clientProcessMiss(clientHttpRequest *);
 static void clientBuildReplyHeader(clientHttpRequest * http, HttpReply * rep);
@@ -77,6 +86,9 @@ static RH clientRedirectDone;
 static STCB clientHandleIMSReply;
 static int clientGetsOldEntry(StoreEntry * new, StoreEntry * old, request_t * request);
 static int checkAccelOnly(clientHttpRequest *);
+#if USE_IDENT
+static IDCB clientIdentDone;
+#endif
 static int clientOnlyIfCached(clientHttpRequest * http);
 static STCB clientSendMoreData;
 static STCB clientCacheHit;
@@ -85,7 +97,6 @@ static void clientInterpretRequestHeaders(clientHttpRequest *);
 static void clientProcessRequest(clientHttpRequest *);
 static void clientProcessExpired(void *data);
 static void clientProcessOnlyIfCachedMiss(clientHttpRequest * http);
-static HttpReply *clientConstructProxyAuthReply(clientHttpRequest * http);
 static int clientCachable(clientHttpRequest * http);
 static int clientHierarchical(clientHttpRequest * http);
 static int clientCheckContentLength(request_t * r);
@@ -108,16 +119,24 @@ checkAccelOnly(clientHttpRequest * http)
     return 1;
 }
 
+#if USE_IDENT
+void
+clientIdentDone(const char *ident, void *data)
+{
+    ConnStateData *conn = data;
+    if (ident)
+	xstrncpy(conn->ident, ident, sizeof(conn->ident));
+    else
+	xstrncpy(conn->ident, "-", sizeof(conn->ident));
+}
+#endif
+
 void
 clientAccessCheck(void *data)
 {
     clientHttpRequest *http = data;
     ConnStateData *conn = http->conn;
     const char *browser;
-    if (Config.onoff.ident_lookup && conn->ident.state == IDENT_NONE) {
-	identStart(-1, conn, clientAccessCheck, http);
-	return;
-    }
     if (checkAccelOnly(http)) {
 	clientAccessCheckDone(0, http);
 	return;
@@ -126,8 +145,17 @@ clientAccessCheck(void *data)
     http->acl_checklist = aclChecklistCreate(Config.accessList.http,
 	http->request,
 	conn->peer.sin_addr,
+	conn->me.sin_addr,
 	browser,
-	conn->ident.ident);
+	conn->ident);
+#if USE_IDENT
+    /*
+     * hack for ident ACL. It needs to get full addresses, and a
+     * place to store the ident result on persistent connections...
+     */
+    http->acl_checklist->conn = conn;
+    cbdataLock(http->acl_checklist->conn);
+#endif
     aclNBCheck(http->acl_checklist, clientAccessCheckDone, http);
 }
 
@@ -144,35 +172,6 @@ clientOnlyIfCached(clientHttpRequest * http)
 	EBIT_TEST(r->cache_control->mask, CC_ONLY_IF_CACHED);
 }
 
-static HttpReply *
-clientConstructProxyAuthReply(clientHttpRequest * http)
-{
-    ErrorState *err;
-    HttpReply *rep;
-    if (!http->flags.accel) {
-	/* Proxy authorisation needed */
-	err = errorCon(ERR_CACHE_ACCESS_DENIED,
-	    HTTP_PROXY_AUTHENTICATION_REQUIRED);
-    } else {
-	/* WWW authorisation needed */
-	err = errorCon(ERR_CACHE_ACCESS_DENIED, HTTP_UNAUTHORIZED);
-    }
-    err->request = requestLink(http->request);
-    rep = errorBuildReply(err);
-    errorStateFree(err);
-    /* add Authenticate header */
-    if (!http->flags.accel) {
-	/* Proxy authorisation needed */
-	httpHeaderPutStrf(&rep->header, HDR_PROXY_AUTHENTICATE,
-	    proxy_auth_challenge_fmt, Config.proxyAuthRealm);
-    } else {
-	/* WWW Authorisation needed */
-	httpHeaderPutStrf(&rep->header, HDR_WWW_AUTHENTICATE,
-	    proxy_auth_challenge_fmt, Config.proxyAuthRealm);
-    }
-    return rep;
-}
-
 StoreEntry *
 clientCreateStoreEntry(clientHttpRequest * h, method_t m, request_flags flags)
 {
@@ -182,25 +181,28 @@ clientCreateStoreEntry(clientHttpRequest * h, method_t m, request_flags flags)
      * so make a fake one.
      */
     if (h->request == NULL)
-	h->request = requestLink(requestCreate(m, PROTO_NONE, NULL));
+	h->request = requestLink(requestCreate(m, PROTO_NONE, null_string));
     e = storeCreateEntry(h->uri, h->log_uri, flags, m);
     storeClientListAdd(e, h);
 #if DELAY_POOLS
-    delaySetStoreClient(e, h, h->request->delay_id);
+    delaySetStoreClient(e, h, delayClient(h->request));
 #endif
     storeClientCopy(e, 0, 0, CLIENT_SOCK_SZ,
 	memAllocate(MEM_CLIENT_SOCK_BUF), clientSendMoreData, h);
     return e;
 }
 
-
 void
 clientAccessCheckDone(int answer, void *data)
 {
     clientHttpRequest *http = data;
     int page_id = -1;
+    http_status status;
     ErrorState *err = NULL;
-    debug(33, 5) ("clientAccessCheckDone: '%s' answer=%d\n", http->uri, answer);
+    debug(33, 2) ("The request %s %s is %s, because it matched '%s'\n",
+	RequestMethodStr[http->request->method], http->uri,
+	answer ? "ALLOWED" : "DENIED",
+	AclMatchedName ? AclMatchedName : "NO ACL's");
     http->acl_checklist = NULL;
     if (answer == ACCESS_ALLOWED) {
 	safe_free(http->uri);
@@ -208,29 +210,38 @@ clientAccessCheckDone(int answer, void *data)
 	assert(http->redirect_state == REDIRECT_NONE);
 	http->redirect_state = REDIRECT_PENDING;
 	redirectStart(http, clientRedirectDone, http);
-    } else if (answer == ACCESS_REQ_PROXY_AUTH) {
-	http->log_type = LOG_TCP_DENIED;
-	http->entry = clientCreateStoreEntry(http, http->request->method,
-	    null_request_flags);
-	/* create appropriate response */
-	http->entry->mem_obj->reply = clientConstructProxyAuthReply(http);
-	httpReplySwapOut(http->entry->mem_obj->reply, http->entry);
-	storeComplete(http->entry);
     } else {
 	debug(33, 5) ("Access Denied: %s\n", http->uri);
 	debug(33, 5) ("AclMatchedName = %s\n",
 	    AclMatchedName ? AclMatchedName : "<null>");
+	/*
+	 * NOTE: get page_id here, based on AclMatchedName because
+	 * if USE_DELAY_POOLS is enabled, then AclMatchedName gets
+	 * clobbered in the clientCreateStoreEntry() call
+	 * just below.  Pedro Ribeiro <pribeiro@isel.pt>
+	 */
+	page_id = aclGetDenyInfoPage(&Config.denyInfoList, AclMatchedName);
 	http->log_type = LOG_TCP_DENIED;
 	http->entry = clientCreateStoreEntry(http, http->request->method,
 	    null_request_flags);
-	page_id = aclGetDenyInfoPage(&Config.denyInfoList, AclMatchedName);
-	/* NOTE: don't use HTTP_UNAUTHORIZED because then the
-	 * stupid browser wants us to authenticate */
-	err = errorCon(ERR_ACCESS_DENIED, HTTP_FORBIDDEN);
+	if (answer == ACCESS_REQ_PROXY_AUTH || aclIsProxyAuth(AclMatchedName)) {
+	    if (!http->flags.accel) {
+		/* Proxy authorisation needed */
+		status = HTTP_PROXY_AUTHENTICATION_REQUIRED;
+	    } else {
+		/* WWW authorisation needed */
+		status = HTTP_UNAUTHORIZED;
+	    }
+	    if (page_id <= 0)
+		page_id = ERR_CACHE_ACCESS_DENIED;
+	} else {
+	    status = HTTP_FORBIDDEN;
+	    if (page_id <= 0)
+		page_id = ERR_ACCESS_DENIED;
+	}
+	err = errorCon(page_id, status);
 	err->request = requestLink(http->request);
 	err->src_addr = http->conn->peer.sin_addr;
-	if (page_id > 0)
-	    err->page_id = page_id;
 	errorAppendEntry(http->entry, err);
     }
 }
@@ -265,6 +276,7 @@ clientRedirectDone(void *data, char *result)
 	new_request->http_ver = old_request->http_ver;
 	httpHeaderAppend(&new_request->header, &old_request->header);
 	new_request->client_addr = old_request->client_addr;
+	new_request->my_addr = old_request->my_addr;
 	new_request->flags.redirected = 1;
 	if (old_request->body) {
 	    new_request->body = xmalloc(old_request->body_sz);
@@ -298,16 +310,21 @@ clientProcessExpired(void *data)
     }
     http->request->flags.refresh = 1;
     http->old_entry = http->entry;
+    /*
+     * Assert that 'http' is already a client of old_entry.  If 
+     * it is not, then the beginning of the object data might get
+     * freed from memory before we need to access it.
+     */
+    assert(storeClientListSearch(http->old_entry->mem_obj, http));
     entry = storeCreateEntry(url,
 	http->log_uri,
 	http->request->flags,
 	http->request->method);
     /* NOTE, don't call storeLockObject(), storeCreateEntry() does it */
     storeClientListAdd(entry, http);
-    storeClientListAdd(http->old_entry, http);
 #if DELAY_POOLS
-    delaySetStoreClient(entry, http, http->request->delay_id);
-    delaySetStoreClient(http->old_entry, http, http->request->delay_id);
+    /* delay_id is already set on original store client */
+    delaySetStoreClient(entry, http, delayClient(http->request));
 #endif
     entry->lastmod = http->old_entry->lastmod;
     debug(33, 5) ("clientProcessExpired: lastmod %d\n", (int) entry->lastmod);
@@ -315,10 +332,10 @@ clientProcessExpired(void *data)
     http->entry = entry;
     http->out.offset = 0;
     fwdStart(http->conn->fd, http->entry, http->request,
-	http->conn->peer.sin_addr);
+	http->conn->peer.sin_addr, http->conn->me.sin_addr);
     /* Register with storage manager to receive updates when data comes in. */
-    if (entry->store_status == STORE_ABORTED)
-	debug(33, 0) ("clientProcessExpired: entry->swap_status == STORE_ABORTED\n");
+    if (EBIT_TEST(entry->flags, ENTRY_ABORTED))
+	debug(33, 0) ("clientProcessExpired: found ENTRY_ABORTED object\n");
     storeClientCopy(entry,
 	http->out.offset,
 	http->out.offset,
@@ -336,6 +353,12 @@ clientGetsOldEntry(StoreEntry * new_entry, StoreEntry * old_entry, request_t * r
 	debug(33, 5) ("clientGetsOldEntry: YES, broken HTTP reply\n");
 	return 1;
     }
+    /* If the reply is a failure then send the old object as a last
+     * resort */
+    if (status >= 500 && status < 600) {
+	debug(33, 2) ("clientGetsOldEntry: YES, failure reply=%d\n", status);
+	return 1;
+    }
     /* If the reply is anything but "Not Modified" then
      * we must forward it to the client */
     if (HTTP_NOT_MODIFIED != status) {
@@ -365,46 +388,24 @@ clientHandleIMSReply(void *data, char *buf, ssize_t size)
 {
     clientHttpRequest *http = data;
     StoreEntry *entry = http->entry;
-    MemObject *mem = entry->mem_obj;
+    MemObject *mem;
     const char *url = storeUrl(entry);
     int unlink_request = 0;
     StoreEntry *oldentry;
     int recopy = 1;
-    const http_status status = mem->reply->sline.status;
+    http_status status;
     debug(33, 3) ("clientHandleIMSReply: %s, %d bytes\n", url, (int) size);
-    if (size < 0 && entry->store_status != STORE_ABORTED)
-#if OLD_CODE
-	/*
-	 * This storeAbort() call causes wrong behaviour in some situations.
-	 * If there are multiple clients hanging on this entry, the second
-	 * client will not want the entry to be aborted.  This shows up in
-	 * the stack trace below:
-	 *
-	 * #0  0xef5f452c in kill ()
-	 * #1  0xef5ba620 in abort ()
-	 * #2  0x7d680 in __eprintf (string=0x82a70 "%s:%u: failed assertion `%s'\n", expression=0x82a90 "client_side.c", line=1111, filename=0x83468 "size > 0")
-	 * #3  0x2ad7c in clientCacheHit (data=0x31ae318, buf=0x17f1450 "", size=0) at client_side.c:1111
-	 * #4  0x65cd4 in storeClientCopy2 (e=0x4c48c0, sc=0x2eedd70) at store_client.c:264
-	 * #5  0x665b0 in InvokeHandlers (e=0x4c48c0) at store_client.c:510
-	 * #6  0x63c90 in storeAbort (e=0x4c48c0, cbflag=1) at store.c:626
-	 * #7  0x29190 in clientHandleIMSReply (data=0x31e9518, buf=0x2268810 "", size=-1) at client_side.c:343
-	 * #8  0x66450 in storeUnregister (e=0x2910c, data=0x31e9518) at store_client.c:467
-	 * #9  0x29a54 in httpRequestFree (data=0x31e9518) at client_side.c:579
-	 * #10 0x29eb4 in connStateFree (fd=534528, data=0x314d0d0) at client_side.c:667
-	 * #11 0x2f27c in commCallCloseHandlers (fd=42) at comm.c:501
-	 * #12 0x2f410 in comm_close (fd=42) at comm.c:565
-	 * #13 0x2d054 in clientReadRequest (fd=42, data=0x314d0d0) at client_side.c:2038
-	 * #14 0x309c8 in comm_poll (msec=434) at comm_select.c:354
-	 * #15 0x4da64 in main (argc=5, argv=0xeffffd4c) at main.c:587
-	 */
-	storeAbort(entry, 1);
-#endif
-    /*
-     * Lets just try to return here and see what kind of problems that
-     * causes
-     */
-    return;
-    if (entry->store_status == STORE_ABORTED) {
+    if (entry == NULL) {
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
+	return;
+    }
+    if (size < 0 && !EBIT_TEST(entry->flags, ENTRY_ABORTED)) {
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
+	return;
+    }
+    mem = entry->mem_obj;
+    status = mem->reply->sline.status;
+    if (EBIT_TEST(entry->flags, ENTRY_ABORTED)) {
 	debug(33, 3) ("clientHandleIMSReply: ABORTED '%s'\n", url);
 	/* We have an existing entry, but failed to validate it */
 	/* Its okay to send the old one anyway */
@@ -472,7 +473,7 @@ clientHandleIMSReply(void *data, char *buf, ssize_t size)
 	recopy = 0;
     }
     http->old_entry = NULL;	/* done with old_entry */
-    assert(entry->store_status != STORE_ABORTED);
+    assert(!EBIT_TEST(entry->flags, ENTRY_ABORTED));
     if (recopy) {
 	storeClientCopy(entry,
 	    http->out.offset,
@@ -634,21 +635,22 @@ httpRequestFree(void *data)
     clientHttpRequest *http = data;
     clientHttpRequest **H;
     ConnStateData *conn = http->conn;
-    StoreEntry *entry = http->entry;
+    StoreEntry *e;
     request_t *request = http->request;
     MemObject *mem = NULL;
-    debug(33, 3) ("httpRequestFree: %s\n", storeUrl(entry));
+    debug(33, 3) ("httpRequestFree: %s\n", storeUrl(http->entry));
     if (!clientCheckTransferDone(http)) {
-	if (entry)
-	    storeUnregister(entry, http);	/* unregister BEFORE abort */
-	CheckQuickAbort(http);
-	entry = http->entry;	/* reset, IMS might have changed it */
-	if (entry && entry->ping_status == PING_WAITING)
-	    storeReleaseRequest(entry);
+	if ((e = http->entry)) {
+	    http->entry = NULL;
+	    storeUnregister(e, http);
+	    storeUnlockObject(e);
+	}
+	if (http->entry && http->entry->ping_status == PING_WAITING)
+	    storeReleaseRequest(http->entry);
     }
     assert(http->log_type < LOG_TYPE_MAX);
-    if (entry)
-	mem = entry->mem_obj;
+    if (http->entry)
+	mem = http->entry->mem_obj;
     if (http->out.size || http->log_type) {
 	http->al.icp.opcode = ICP_INVALID;
 	http->al.url = http->log_uri;
@@ -664,7 +666,7 @@ httpRequestFree(void *data)
 	if (request->user_ident[0])
 	    http->al.cache.ident = request->user_ident;
 	else
-	    http->al.cache.ident = conn->ident.ident;
+	    http->al.cache.ident = conn->ident;
 	if (request) {
 	    Packer p;
 	    MemBuf mb;
@@ -692,17 +694,17 @@ httpRequestFree(void *data)
     safe_free(http->al.headers.reply);
     safe_free(http->redirect.location);
     stringClean(&http->range_iter.boundary);
-    if (entry) {
+    if ((e = http->entry)) {
 	http->entry = NULL;
-	storeUnregister(entry, http);
-	storeUnlockObject(entry);
+	storeUnregister(e, http);
+	storeUnlockObject(e);
     }
     /* old_entry might still be set if we didn't yet get the reply
      * code in clientHandleIMSReply() */
-    if (http->old_entry) {
-	storeUnregister(http->old_entry, http);
-	storeUnlockObject(http->old_entry);
+    if ((e = http->old_entry)) {
 	http->old_entry = NULL;
+	storeUnregister(e, http);
+	storeUnlockObject(e);
     }
     requestUnlink(http->request);
     assert(http != http->next);
@@ -716,6 +718,7 @@ httpRequestFree(void *data)
     assert(*H != NULL);
     *H = http->next;
     http->next = NULL;
+    dlinkDelete(&http->active, &ClientActiveRequests);
     cbdataFree(http);
 }
 
@@ -732,8 +735,6 @@ connStateFree(int fd, void *data)
 	assert(connState->chr != connState->chr->next);
 	httpRequestFree(http);
     }
-    if (connState->ident.fd > -1)
-	comm_close(connState->ident.fd);
     safe_free(connState->in.buf);
     /* XXX account connState->in.buf */
     pconnHistCount(0, connState->nrequests);
@@ -752,6 +753,7 @@ clientInterpretRequestHeaders(clientHttpRequest * http)
 {
     request_t *request = http->request;
     const HttpHeader *req_hdr = &request->header;
+    int no_cache = 0;
 #if USE_USERAGENT_LOG
     const char *str;
 #endif
@@ -761,18 +763,23 @@ clientInterpretRequestHeaders(clientHttpRequest * http)
 	request->flags.ims = 1;
     if (httpHeaderHas(req_hdr, HDR_PRAGMA)) {
 	String s = httpHeaderGetList(req_hdr, HDR_PRAGMA);
-	if (strListIsMember(&s, "no-cache", ',')) {
-#if HTTP_VIOLATIONS
-	    if (Config.onoff.reload_into_ims)
-		request->flags.nocache_hack = 1;
-	    else if (refresh_nocache_hack)
-		request->flags.nocache_hack = 1;
-	    else
-#endif
-		request->flags.nocache = 1;
-	}
+	if (strListIsMember(&s, "no-cache", ','))
+	    no_cache++;
 	stringClean(&s);
     }
+    if (request->cache_control)
+	if (EBIT_TEST(request->cache_control->mask, CC_NO_CACHE))
+	    no_cache++;
+    if (no_cache) {
+#if HTTP_VIOLATIONS
+	if (Config.onoff.reload_into_ims)
+	    request->flags.nocache_hack = 1;
+	else if (refresh_nocache_hack)
+	    request->flags.nocache_hack = 1;
+	else
+#endif
+	    request->flags.nocache = 1;
+    }
     /* ignore range header in non-GETs */
     if (request->method == METHOD_GET) {
 	request->range = httpHeaderGetRange(req_hdr);
@@ -785,8 +792,12 @@ clientInterpretRequestHeaders(clientHttpRequest * http)
 	request->flags.auth = 1;
     if (httpHeaderHas(req_hdr, HDR_VIA)) {
 	String s = httpHeaderGetList(req_hdr, HDR_VIA);
-	/* ThisCache cannot be a member of Via header, "1.0 ThisCache" can */
-	if (strListIsSubstr(&s, ThisCache, ',')) {
+	/*
+	 * ThisCache cannot be a member of Via header, "1.0 ThisCache" can.
+	 * Note ThisCache2 has a space prepended to the hostname so we don't
+	 * accidentally match super-domains.
+	 */
+	if (strListIsSubstr(&s, ThisCache2, ',')) {
 	    debugObj(33, 1, "WARNING: Forwarding loop detected for:\n",
 		request, (ObjPackMethod) & httpRequestPack);
 	    request->flags.loopdetect = 1;
@@ -815,13 +826,6 @@ clientInterpretRequestHeaders(clientHttpRequest * http)
 	request->flags.cachable = 1;
     if (clientHierarchical(http))
 	request->flags.hierarchical = 1;
-#if DELAY_POOLS
-    if (delayClient(http)) {
-	debug(33, 5) ("clientInterpretRequestHeaders: delay request class %d position %d\n",
-	    request->delay_id >> 16,
-	    request->delay_id & 0xFFFF);
-    }
-#endif
     debug(33, 5) ("clientInterpretRequestHeaders: REQ_NOCACHE = %s\n",
 	request->flags.nocache ? "SET" : "NOT SET");
     debug(33, 5) ("clientInterpretRequestHeaders: REQ_CACHABLE = %s\n",
@@ -847,9 +851,6 @@ clientSetKeepaliveFlag(clientHttpRequest * http)
 	RequestMethodStr[request->method]);
     if (httpMsgIsPersistent(request->http_ver, req_hdr))
 	request->flags.proxy_keepalive = 1;
-    if (request->method == METHOD_POST || request->method == METHOD_PUT)
-	if (!Config.onoff.persistent_client_posts)
-	    request->flags.proxy_keepalive = 0;
 }
 
 static int
@@ -880,6 +881,7 @@ clientCachable(clientHttpRequest * http)
      * This may not work yet for 'dst' and 'dst_domain' ACLs.
      */
     ch.src_addr = http->conn->peer.sin_addr;
+    ch.my_addr = http->conn->me.sin_addr;
     ch.request = http->request;
     /*
      * aclCheckFast returns 1 for ALLOW and 0 for DENY.  The default
@@ -1188,12 +1190,12 @@ clientCacheHit(void *data, char *buf, ssize_t size)
     request_t *r = http->request;
     debug(33, 3) ("clientCacheHit: %s, %d bytes\n", http->uri, (int) size);
     if (http->entry == NULL) {
-	memFree(MEM_CLIENT_SOCK_BUF, buf);
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
 	debug(33, 3) ("clientCacheHit: request aborted\n");
 	return;
     } else if (size < 0) {
 	/* swap in failure */
-	memFree(MEM_CLIENT_SOCK_BUF, buf);
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
 	debug(33, 3) ("clientCacheHit: swapin failure for %s\n", http->uri);
 	http->log_type = LOG_TCP_SWAPFAIL_MISS;
 	if ((e = http->entry)) {
@@ -1206,17 +1208,17 @@ clientCacheHit(void *data, char *buf, ssize_t size)
     }
     assert(size > 0);
     mem = e->mem_obj;
-    assert(e->store_status != STORE_ABORTED);
+    assert(!EBIT_TEST(e->flags, ENTRY_ABORTED));
     if (mem->reply->sline.status == 0) {
 	/*
 	 * we don't have full reply headers yet; either wait for more or
 	 * punt to clientProcessMiss.
 	 */
 	if (e->mem_status == IN_MEMORY || e->store_status == STORE_OK) {
-	    memFree(MEM_CLIENT_SOCK_BUF, buf);
+	    memFree(buf, MEM_CLIENT_SOCK_BUF);
 	    clientProcessMiss(http);
 	} else if (size == CLIENT_SOCK_SZ && http->out.offset == 0) {
-	    memFree(MEM_CLIENT_SOCK_BUF, buf);
+	    memFree(buf, MEM_CLIENT_SOCK_BUF);
 	    clientProcessMiss(http);
 	} else {
 	    debug(33, 3) ("clientCacheHit: waiting for HTTP reply headers\n");
@@ -1288,7 +1290,7 @@ clientCacheHit(void *data, char *buf, ssize_t size)
 	    http->log_type = LOG_TCP_MISS;
 	    clientProcessMiss(http);
 	}
-	memFree(MEM_CLIENT_SOCK_BUF, buf);
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
     } else if (r->flags.ims) {
 	/*
 	 * Handle If-Modified-Since requests from the client
@@ -1296,7 +1298,8 @@ clientCacheHit(void *data, char *buf, ssize_t size)
 	if (mem->reply->sline.status != HTTP_OK) {
 	    debug(33, 4) ("clientCacheHit: Reply code %d != 200\n",
 		mem->reply->sline.status);
-	    memFree(MEM_CLIENT_SOCK_BUF, buf);
+	    memFree(buf, MEM_CLIENT_SOCK_BUF);
+	    http->log_type = LOG_TCP_MISS;
 	    clientProcessMiss(http);
 	} else if (modifiedSince(e, http->request)) {
 	    http->log_type = LOG_TCP_IMS_HIT;
@@ -1304,7 +1307,7 @@ clientCacheHit(void *data, char *buf, ssize_t size)
 	} else {
 	    MemBuf mb = httpPacked304Reply(e->mem_obj->reply);
 	    http->log_type = LOG_TCP_IMS_HIT;
-	    memFree(MEM_CLIENT_SOCK_BUF, buf);
+	    memFree(buf, MEM_CLIENT_SOCK_BUF);
 	    storeUnregister(e, http);
 	    storeUnlockObject(e);
 	    e = clientCreateStoreEntry(http, http->request->method, null_request_flags);
@@ -1467,27 +1470,29 @@ clientSendMoreData(void *data, char *buf, ssize_t size)
     debug(33, 5) ("clientSendMoreData: %s, %d bytes\n", http->uri, (int) size);
     assert(size <= CLIENT_SOCK_SZ);
     assert(http->request != NULL);
+    dlinkDelete(&http->active, &ClientActiveRequests);
+    dlinkAdd(http, &http->active, &ClientActiveRequests);
     debug(33, 5) ("clientSendMoreData: FD %d '%s', out.offset=%d \n",
 	fd, storeUrl(entry), (int) http->out.offset);
     if (conn->chr != http) {
 	/* there is another object in progress, defer this one */
 	debug(33, 1) ("clientSendMoreData: Deferring %s\n", storeUrl(entry));
-	memFree(MEM_CLIENT_SOCK_BUF, buf);
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
 	return;
-    } else if (entry && entry->store_status == STORE_ABORTED) {
+    } else if (entry && EBIT_TEST(entry->flags, ENTRY_ABORTED)) {
 	/* call clientWriteComplete so the client socket gets closed */
 	clientWriteComplete(fd, NULL, 0, COMM_OK, http);
-	memFree(MEM_CLIENT_SOCK_BUF, buf);
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
 	return;
     } else if (size < 0) {
 	/* call clientWriteComplete so the client socket gets closed */
 	clientWriteComplete(fd, NULL, 0, COMM_OK, http);
-	memFree(MEM_CLIENT_SOCK_BUF, buf);
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
 	return;
     } else if (size == 0) {
 	/* call clientWriteComplete so the client socket gets closed */
 	clientWriteComplete(fd, NULL, 0, COMM_OK, http);
-	memFree(MEM_CLIENT_SOCK_BUF, buf);
+	memFree(buf, MEM_CLIENT_SOCK_BUF);
 	return;
     }
     if (http->out.offset == 0) {
@@ -1568,7 +1573,7 @@ clientSendMoreData(void *data, char *buf, ssize_t size)
     /* write */
     comm_write_mbuf(fd, mb, clientWriteComplete, http);
     /* if we don't do it, who will? */
-    memFree(MEM_CLIENT_SOCK_BUF, buf);
+    memFree(buf, MEM_CLIENT_SOCK_BUF);
 }
 
 static void
@@ -1601,8 +1606,8 @@ clientKeepaliveNextRequest(clientHttpRequest * http)
 	    conn->fd);
 	assert(entry);
 	if (0 == storeClientCopyPending(entry, http)) {
-	    if (entry->store_status == STORE_ABORTED)
-		debug(33, 0) ("clientKeepaliveNextRequest: entry->swap_status == STORE_ABORTED\n");
+	    if (EBIT_TEST(entry->flags, ENTRY_ABORTED))
+		debug(33, 0) ("clientKeepaliveNextRequest: ENTRY_ABORTED\n");
 	    storeClientCopy(entry,
 		http->out.offset,
 		http->out.offset,
@@ -1635,7 +1640,7 @@ clientWriteComplete(int fd, char *bufnotused, size_t size, int errflag, void *da
 	comm_close(fd);
     } else if (NULL == entry) {
 	comm_close(fd);		/* yuk */
-    } else if (entry->store_status == STORE_ABORTED) {
+    } else if (EBIT_TEST(entry->flags, ENTRY_ABORTED)) {
 	comm_close(fd);
     } else if ((done = clientCheckTransferDone(http)) != 0 || size == 0) {
 	debug(33, 5) ("clientWriteComplete: FD %d transfer is DONE\n", fd);
@@ -1658,8 +1663,8 @@ clientWriteComplete(int fd, char *bufnotused, size_t size, int errflag, void *da
     } else {
 	/* More data will be coming from primary server; register with 
 	 * storage manager. */
-	if (entry->store_status == STORE_ABORTED)
-	    debug(33, 0) ("clientWriteComplete 2: entry->swap_status == STORE_ABORTED\n");
+	if (EBIT_TEST(entry->flags, ENTRY_ABORTED))
+	    debug(33, 0) ("clientWriteComplete 2: ENTRY_ABORTED\n");
 	storeClientCopy(entry,
 	    http->out.offset,
 	    http->out.offset,
@@ -1817,7 +1822,7 @@ clientProcessRequest(clientHttpRequest * http)
 	storeCreateMemObject(http->entry, http->uri, http->log_uri);
 	storeClientListAdd(http->entry, http);
 #if DELAY_POOLS
-	delaySetStoreClient(http->entry, http, http->request->delay_id);
+	delaySetStoreClient(http->entry, http, delayClient(r));
 #endif
 	http->entry->refcount++;
 	storeClientCopy(http->entry,
@@ -1886,20 +1891,22 @@ clientProcessMiss(clientHttpRequest * http)
     }
     if (http->flags.internal)
 	r->protocol = PROTO_INTERNAL;
-    fwdStart(http->conn->fd, http->entry, r, http->conn->peer.sin_addr);
+    fwdStart(http->conn->fd, http->entry, r,
+	http->conn->peer.sin_addr, http->conn->me.sin_addr);
 }
 
 static clientHttpRequest *
 parseHttpRequestAbort(ConnStateData * conn, const char *uri)
 {
     clientHttpRequest *http = xcalloc(1, sizeof(clientHttpRequest));
-    cbdataAdd(http, MEM_NONE);
+    cbdataAdd(http, cbdataXfree, 0);
     http->conn = conn;
     http->start = current_time;
     http->req_sz = conn->in.offset;
     http->uri = xstrdup(uri);
     http->log_uri = xstrndup(uri, MAX_URL);
     http->range_iter.boundary = StringNull;
+    dlinkAdd(http, &http->active, &ClientActiveRequests);
     return http;
 }
 
@@ -1975,19 +1982,19 @@ parseHttpRequest(ConnStateData * conn, method_t * method_p, int *status,
 	debug(33, 1) ("parseHttpRequest: Missing URL\n");
 	return parseHttpRequestAbort(conn, "error:missing-url");
     }
-    while (isspace(*url))
+    while (xisspace(*url))
 	url++;
     t = url + strlen(url);
     assert(*t == '\0');
     token = NULL;
     while (t > url) {
 	t--;
-	if (isspace(*t) && !strncmp(t + 1, "HTTP/", 5)) {
+	if (xisspace(*t) && !strncmp(t + 1, "HTTP/", 5)) {
 	    token = t + 1;
 	    break;
 	}
     }
-    while (t > url && isspace(*t))
+    while (t > url && xisspace(*t))
 	*(t--) = '\0';
     debug(33, 5) ("parseHttpRequest: URI is '%s'\n", url);
     if (token == NULL) {
@@ -2024,7 +2031,7 @@ parseHttpRequest(ConnStateData * conn, method_t * method_p, int *status,
 
     /* Ok, all headers are received */
     http = xcalloc(1, sizeof(clientHttpRequest));
-    cbdataAdd(http, MEM_NONE);
+    cbdataAdd(http, cbdataXfree, 0);
     http->http_ver = http_ver;
     http->conn = conn;
     http->start = current_time;
@@ -2033,6 +2040,7 @@ parseHttpRequest(ConnStateData * conn, method_t * method_p, int *status,
     *prefix_p = xmalloc(prefix_sz + 1);
     xmemcpy(*prefix_p, conn->in.buf, prefix_sz);
     *(*prefix_p + prefix_sz) = '\0';
+    dlinkAdd(http, &http->active, &ClientActiveRequests);
 
     debug(33, 5) ("parseHttpRequest: Request Header is\n%s\n", (*prefix_p) + *req_line_sz_p);
     if ((t = strchr(url, '#')))	/* remove HTML anchors */
@@ -2201,7 +2209,7 @@ clientReadRequest(int fd, void *data)
     while (conn->in.offset > 0) {
 	int nrequests;
 	size_t req_line_sz;
-	while (conn->in.offset > 0 && isspace(conn->in.buf[0])) {
+	while (conn->in.offset > 0 && xisspace(conn->in.buf[0])) {
 	    xmemmove(conn->in.buf, conn->in.buf + 1, conn->in.offset - 1);
 	    conn->in.offset--;
 	}
@@ -2270,9 +2278,9 @@ clientReadRequest(int fd, void *data)
 	    request->flags.accelerated = http->flags.accel;
 	    if (!http->flags.internal) {
 		if (internalCheck(strBuf(request->urlpath))) {
-		    if (0 == strcasecmp(request->host, internalHostname())) {
-			if (request->port == Config.Port.http->i)
-			    http->flags.internal = 1;
+		    if (0 == strcasecmp(request->host, internalHostname()) &&
+			request->port == Config.Port.http->i) {
+			http->flags.internal = 1;
 		    } else if (internalStaticCheck(strBuf(request->urlpath))) {
 			xstrncpy(request->host, internalHostname(), SQUIDHOSTNAMELEN);
 			request->port = Config.Port.http->i;
@@ -2285,6 +2293,7 @@ clientReadRequest(int fd, void *data)
 	    safe_free(http->log_uri);
 	    http->log_uri = xstrdup(urlCanonicalClean(request));
 	    request->client_addr = conn->peer.sin_addr;
+	    request->my_addr = conn->me.sin_addr;
 	    request->http_ver = http->http_ver;
 	    if (!urlCheckRequest(request)) {
 		err = errorCon(ERR_UNSUP_REQ, HTTP_NOT_IMPLEMENTED);
@@ -2318,9 +2327,7 @@ clientReadRequest(int fd, void *data)
 	     */
 	    if (request->method != METHOD_GET) {
 		int cont_len = httpHeaderGetInt(&request->header, HDR_CONTENT_LENGTH);
-		int copy_len = conn->in.offset;
-		if (cont_len < copy_len && request->flags.proxy_keepalive)
-		    copy_len = cont_len;
+		int copy_len = XMIN(conn->in.offset, cont_len);
 		if (copy_len > 0) {
 		    assert(conn->in.offset >= copy_len);
 		    request->body_sz = copy_len;
@@ -2437,6 +2444,9 @@ httpAccept(int sock, void *data)
     struct sockaddr_in peer;
     struct sockaddr_in me;
     int max = INCOMING_HTTP_MAX;
+#if USE_IDENT
+    static aclCheck_t identChecklist;
+#endif
     commSetSelect(sock, COMM_SELECT_READ, httpAccept, NULL, 0);
     while (max-- && !httpAcceptDefer()) {
 	memset(&peer, '\0', sizeof(struct sockaddr_in));
@@ -2454,79 +2464,26 @@ httpAccept(int sock, void *data)
 	connState->log_addr.s_addr &= Config.Addrs.client_netmask.s_addr;
 	connState->me = me;
 	connState->fd = fd;
-	connState->ident.fd = -1;
 	connState->in.size = REQUEST_BUF_SIZE;
 	connState->in.buf = xcalloc(connState->in.size, 1);
-	cbdataAdd(connState, MEM_NONE);
+	cbdataAdd(connState, cbdataXfree, 0);
 	/* XXX account connState->in.buf */
 	comm_add_close_handler(fd, connStateFree, connState);
 	if (Config.onoff.log_fqdn)
 	    fqdncache_gethostbyaddr(peer.sin_addr, FQDN_LOOKUP_IF_MISS);
 	commSetTimeout(fd, Config.Timeout.request, requestTimeout, connState);
+#if USE_IDENT
+	identChecklist.src_addr = peer.sin_addr;
+	identChecklist.my_addr = me.sin_addr;
+	if (aclCheckFast(Config.accessList.identLookup, &identChecklist))
+	    identStart(&me, &peer, clientIdentDone, connState);
+#endif
 	commSetSelect(fd, COMM_SELECT_READ, clientReadRequest, connState, 0);
 	commSetDefer(fd, clientReadDefer, connState);
 	(*N)++;
     }
 }
 
-/* return 1 if the request should be aborted */
-static int
-CheckQuickAbort2(const clientHttpRequest * http)
-{
-    int curlen;
-    int minlen;
-    int expectlen;
-
-    if (!http->request->flags.cachable)
-	return 1;
-    if (EBIT_TEST(http->entry->flags, KEY_PRIVATE))
-	return 1;
-    if (http->entry->mem_obj == NULL)
-	return 1;
-    expectlen = http->entry->mem_obj->reply->content_length;
-    curlen = (int) http->entry->mem_obj->inmem_hi;
-    minlen = (int) Config.quickAbort.min << 10;
-    if (minlen < 0)
-	/* disabled */
-	return 0;
-    if (curlen > expectlen)
-	/* bad content length */
-	return 1;
-    if ((expectlen - curlen) < minlen)
-	/* only little more left */
-	return 0;
-    if ((expectlen - curlen) > (Config.quickAbort.max << 10))
-	/* too much left to go */
-	return 1;
-    if (expectlen < 100)
-	/* avoid FPE */
-	return 0;
-    if ((curlen / (expectlen / 100)) > Config.quickAbort.pct)
-	/* past point of no return */
-	return 0;
-    return 1;
-}
-
-
-static void
-CheckQuickAbort(clientHttpRequest * http)
-{
-    StoreEntry *entry = http->entry;
-    /* Note, set entry here because http->entry might get changed (for IMS
-     * requests) during the storeAbort() call */
-    if (entry == NULL)
-	return;
-    if (storePendingNClients(entry) > 0)
-	return;
-    if (entry->store_status != STORE_PENDING)
-	return;
-    if (CheckQuickAbort2(http) == 0)
-	return;
-    debug(33, 3) ("CheckQuickAbort: ABORTING %s\n", storeUrl(entry));
-    Counter.aborted_requests++;
-    storeAbort(entry, 1);
-}
-
 #define SENDING_BODY 0
 #define SENDING_HDRSONLY 1
 static int
@@ -2546,10 +2503,10 @@ clientCheckTransferDone(clientHttpRequest * http)
     if (http->flags.done_copying)
 	return 1;
     /*
-     * Handle STORE_OK and STORE_ABORTED objects.
+     * Handle STORE_OK objects.
      * objectLen(entry) will be set proprely.
      */
-    if (entry->store_status != STORE_PENDING) {
+    if (entry->store_status == STORE_OK) {
 	if (http->out.offset >= objectLen(entry))
 	    return 1;
 	else
@@ -2579,8 +2536,7 @@ clientCheckTransferDone(clientHttpRequest * http)
     /*
      * Figure out how much data we are supposed to send.
      * If we are sending a body and we don't have a content-length,
-     * then we must wait for the object to become STORE_OK or
-     * STORE_ABORTED.
+     * then we must wait for the object to become STORE_OK.
      */
     if (sending == SENDING_HDRSONLY)
 	sendlen = reply->hdr_sz;
