@@ -1,7 +1,7 @@
 
 
 /*
- * $Id: fqdncache.c,v 1.121.2.2 1998/10/30 17:13:12 wessels Exp $
+ * $Id: fqdncache.c,v 1.126 1998/12/05 00:54:24 wessels Exp $
  *
  * DEBUG: section 35    FQDN Cache
  * AUTHOR: Harvest Derived
@@ -98,7 +98,7 @@ fqdncache_release(fqdncache_entry * f)
     dlinkDelete(&f->lru, &lru_list);
     safe_free(f->name);
     safe_free(f->error_message);
-    memFree(MEM_FQDNCACHE_ENTRY, f);
+    memFree(f, MEM_FQDNCACHE_ENTRY);
 }
 
 /* return match for given name */
@@ -151,7 +151,7 @@ fqdncache_purgelru(void *notused)
 	fqdncache_release(f);
 	removed++;
     }
-    debug(35, 3) ("fqdncache_purgelru: removed %d entries\n", removed);
+    debug(35, 9) ("fqdncache_purgelru: removed %d entries\n", removed);
 }
 
 /* create blank fqdncache_entry */
@@ -213,7 +213,7 @@ fqdncache_call_pending(fqdncache_entry * f)
 	    p->handler((f->status == FQDN_CACHED) ? f->names[0] : NULL,
 		p->handlerData);
 	}
-	memFree(MEM_FQDNCACHE_PENDING, p);
+	memFree(p, MEM_FQDNCACHE_PENDING);
     }
     f->pending_head = NULL;	/* nuke list */
     debug(35, 10) ("fqdncache_call_pending: Called %d handlers.\n", nhandler);
@@ -229,7 +229,7 @@ fqdncacheParse(const char *inbuf)
     static fqdncache_entry f;
     int ttl;
     xstrncpy(buf, inbuf, DNS_INBUF_SZ);
-    debug(35, 5) ("fqdncacheParse: parsing:\n%s", buf);
+    debug(35, 5) ("fqdncacheParse: parsing: {%s}\n", buf);
     memset(&f, '\0', sizeof(f));
     f.expires = squid_curtime;
     f.status = FQDN_NEGATIVE_CACHED;
@@ -366,7 +366,7 @@ fqdncache_nbgethostbyaddr(struct in_addr addr, FQDNH * handler, void *handlerDat
     /* for HIT, PENDING, DISPATCHED we've returned.  For MISS we submit */
     c = xcalloc(1, sizeof(*c));
     c->data = f;
-    cbdataAdd(c, MEM_NONE);
+    cbdataAdd(c, cbdataXfree, 0);
     f->status = FQDN_DISPATCHED;
     fqdncacheLockEntry(f);	/* lock while FQDN_DISPATCHED */
     dnsSubmit(f->name, fqdncacheHandleReply, c);
@@ -554,13 +554,13 @@ fqdncacheFreeEntry(void *data)
     int k;
     while ((p = f->pending_head)) {
 	f->pending_head = p->next;
-	memFree(MEM_FQDNCACHE_PENDING, p);
+	memFree(p, MEM_FQDNCACHE_PENDING);
     }
     for (k = 0; k < (int) f->name_count; k++)
 	safe_free(f->names[k]);
     safe_free(f->name);
     safe_free(f->error_message);
-    memFree(MEM_FQDNCACHE_ENTRY, f);
+    memFree(f, MEM_FQDNCACHE_ENTRY);
 }
 
 void
@@ -658,12 +658,6 @@ snmp_netFqdnFn(variable_list * Var, snint * ErrP)
     case FQDN_GHBN:
 	*(Answer->val.integer) = FqdncacheStats.ghba_calls;
 	break;
-#if DELETE_ME
-    case FQDN_LENG:
-	*(Answer->val.integer) = queue_length;
-	Answer->type = SMI_GAUGE32;
-	break;
-#endif
     default:
 	*ErrP = SNMP_ERR_NOSUCHNAME;
 	snmp_var_free(Answer);
