@@ -1,16 +1,20 @@
 
 /*
- * $Id: squid.h,v 1.95.2.18 1997/10/21 16:08:53 wessels Exp $
+ * $Id: squid.h,v 1.185.2.1 1999/04/12 22:58:20 wessels Exp $
  *
  * AUTHOR: Duane Wessels
  *
  * SQUID Internet Object Cache  http://squid.nlanr.net/Squid/
- * --------------------------------------------------------
+ * ----------------------------------------------------------
  *
  *  Squid is the result of efforts by numerous individuals from the
  *  Internet community.  Development is led by Duane Wessels of the
- *  National Laboratory for Applied Network Research and funded by
- *  the National Science Foundation.
+ *  National Laboratory for Applied Network Research and funded by the
+ *  National Science Foundation.  Squid is Copyrighted (C) 1998 by
+ *  Duane Wessels and the University of California San Diego.  Please
+ *  see the COPYRIGHT file for full details.  Squid incorporates
+ *  software developed and/or copyrighted by other sources.  Please see
+ *  the CREDITS file for full details.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,8 +28,8 @@
  *  
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *  
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
+ *
  */
 
 #ifndef SQUID_H
@@ -64,6 +68,14 @@
 #define FD_SETSIZE SQUID_MAXFD
 #endif
 
+#if PURIFY
+#define assert(EX) ((void)0)
+#elif STDC_HEADERS
+#define assert(EX)  ((EX)?((void)0):xassert( # EX , __FILE__, __LINE__))
+#else
+#define assert(EX)  ((EX)?((void)0):xassert("EX", __FILE__, __LINE__))
+#endif
+
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -164,15 +176,43 @@
 #include <getopt.h>
 #endif
 
-#if defined(USE_POLL) && HAVE_POLL
-#if HAVE_POLL_H
-#include <poll.h>
-#endif /* HAVE_POLL_H */
-#else
-#undef USE_POLL
+#if HAVE_DIRENT_H
+#include <dirent.h>
+#define NAMLEN(dirent) strlen((dirent)->d_name)
+#else /* HAVE_DIRENT_H */
+#define dirent direct
+#define NAMLEN(dirent) (dirent)->d_namlen
+#if HAVE_SYS_NDIR_H
+#include <sys/ndir.h>
+#endif /* HAVE_SYS_NDIR_H */
+#if HAVE_SYS_DIR_H
+#include <sys/dir.h>
+#endif /* HAVE_SYS_DIR_H */
+#if HAVE_NDIR_H
+#include <ndir.h>
+#endif /* HAVE_NDIR_H */
+#endif /* HAVE_DIRENT_H */
+
+#if defined(__QNX__)
+#include <unix.h>
 #endif
 
-#ifdef __STDC__
+/*
+ * We require poll.h before using poll().  If the symbols used
+ * by poll() are defined elsewhere, we will need to make this
+ * a more sophisticated test.
+ *  -- Oskar Pearson <oskar@is.co.za>
+ *  -- Stewart Forster <slf@connect.com.au>
+ */
+#if HAVE_POLL
+#if HAVE_POLL_H
+#include <poll.h>
+#else /* HAVE_POLL_H */
+#undef HAVE_POLL
+#endif /* HAVE_POLL_H */
+#endif /* HAVE_POLL */
+
+#if STDC_HEADERS
 #include <stdarg.h>
 #else
 #include <varargs.h>
@@ -180,6 +220,9 @@
 
 /* Make sure syslog goes after stdarg/varargs */
 #ifdef HAVE_SYSLOG_H
+#ifdef _SQUID_AIX_
+#define _XOPEN_EXTENDED_SOURCE
+#endif
 #include <syslog.h>
 #endif
 
@@ -198,9 +241,27 @@
 #define MAXPATHLEN SQUID_MAXPATHLEN
 #endif
 
-#if !defined(HAVE_GETRUSAGE) && defined(_SQUID_HPUX_)
+#if !HAVE_GETRUSAGE
+#if defined(_SQUID_HPUX_)
 #define HAVE_GETRUSAGE 1
 #define getrusage(a, b)  syscall(SYS_GETRUSAGE, a, b)
+#endif
+#endif
+
+#if !HAVE_STRUCT_RUSAGE
+/*
+ * If we don't have getrusage() then we create a fake structure
+ * with only the fields Squid cares about.  This just makes the
+ * source code cleaner, so we don't need lots of #ifdefs in other
+ * places
+ */
+struct rusage {
+    struct timeval ru_stime;
+    struct timeval ru_utime;
+    int ru_maxrss;
+    int ru_majflt;
+};
+
 #endif
 
 #if !defined(HAVE_GETPAGESIZE) && defined(_SQUID_HPUX_)
@@ -212,6 +273,7 @@
 #define BUFSIZ  4096		/* make reasonable guess */
 #endif
 
+
 #ifndef SA_RESTART
 #define SA_RESTART 0
 #endif
@@ -226,29 +288,6 @@
 #define SA_RESETHAND SA_ONESHOT
 #endif
 
-typedef struct sentry StoreEntry;
-typedef struct mem_hdr *mem_ptr;
-typedef struct _peer peer;
-typedef struct icp_common_s icp_common_t;
-typedef struct _cacheinfo cacheinfo;
-typedef struct _aclCheck_t aclCheck_t;
-typedef struct _request request_t;
-typedef struct _MemObject MemObject;
-typedef struct _cachemgr_passwd cachemgr_passwd;
-
-/* 32 bit integer compatability hack */
-#if SIZEOF_INT == 4
-typedef int num32;
-typedef unsigned int u_num32;
-#elif SIZEOF_LONG == 4
-typedef long num32;
-typedef unsigned long u_num32;
-#else
-typedef long num32;		/* assume that long's are 32bit */
-typedef unsigned long u_num32;
-#endif
-#define NUM32LEN sizeof(num32)	/* this should always be 4 */
-
 #if PURIFY
 #define LOCAL_ARRAY(type,name,size) \
         static type *local_##name=NULL; \
@@ -258,124 +297,97 @@ typedef unsigned long u_num32;
 #define LOCAL_ARRAY(type,name,size) static type name[size]
 #endif
 
+#if CBDATA_DEBUG
+#define cbdataAdd(a,b,c)	cbdataAddDbg(a,b,c,__FILE__,__LINE__)
+#endif
+
+#if USE_LEAKFINDER
+#define leakAdd(p) leakAddFL(p,__FILE__,__LINE__)
+#define leakTouch(p) leakTouchFL(p,__FILE__,__LINE__)
+#else
+#define leakAdd(p) p
+#define leakTouch(p) p
+#define leakFree(p) p
+#endif
+
 #if defined(_SQUID_NEXT_) && !defined(S_ISDIR)
 #define S_ISDIR(mode) (((mode) & (_S_IFMT)) == (_S_IFDIR))
 #endif
 
-#include "ansiproto.h"
-
 #ifdef USE_GNUREGEX
 #include "GNUregex.h"
 #elif HAVE_REGEX_H
 #include <regex.h>
 #endif
 
-typedef void (*SIH) (int, void *);	/* swap in */
-typedef int (*QS) (const void *, const void *);
+#if USE_ASYNC_IO
+#undef USE_UNLINKD
+#else
+#define USE_UNLINKD 1
+#endif
+
+#include "md5.h"
+#include "Stack.h"
+
+/* Needed for poll() on Linux at least */
+#if HAVE_POLL
+#ifndef POLLRDNORM
+#define POLLRDNORM POLLIN
+#endif
+#ifndef POLLWRNORM
+#define POLLWRNORM POLLOUT
+#endif
+#endif
+
+#ifdef SQUID_SNMP
+#include "cache_snmp.h"
+#endif
 
-#include "cache_cf.h"
-#include "comm.h"
-#include "debug.h"
-#include "fdstat.h"
-#include "disk.h"
-#include "filemap.h"
 #include "hash.h"
-#include "proto.h"		/* must go before neighbors.h */
-#include "neighbors.h"		/* must go before url.h */
-#include "url.h"
-#include "icp.h"
-#include "errorpage.h"		/* must go after icp.h */
-#include "dns.h"
-#include "ipcache.h"
-#include "fqdncache.h"
-#include "mime.h"
-#include "stack.h"
-#include "stat.h"
-#include "stmem.h"
-#include "store.h"
-#include "tools.h"
-#include "http.h"
-#include "ftp.h"
-#include "gopher.h"
+
+#include "defines.h"
+#include "enums.h"
+#include "typedefs.h"
+#include "structs.h"
+#include "protos.h"
+#include "globals.h"
+
 #include "util.h"
-#include "event.h"
-#include "acl.h"
-#include "async_io.h"
-#include "redirect.h"
-#include "client_side.h"
-#include "useragent.h"
-#include "icmp.h"
-#include "net_db.h"
-#include "client_db.h"
-#include "objcache.h"
-#include "refresh.h"
-#include "unlinkd.h"
+#include "radix.h"
 
 #if !HAVE_TEMPNAM
 #include "tempnam.h"
 #endif
 
-extern void serverConnectionsClose _PARAMS((void));
-extern void shut_down _PARAMS((int));
+#if !HAVE_SNPRINTF
+#include "snprintf.h"
+#endif
 
+#define XMIN(x,y) ((x)<(y)? (x) : (y))
+#define XMAX(a,b) ((a)>(b)? (a) : (b))
 
-extern time_t squid_starttime;	/* main.c */
-extern int do_reuse;		/* main.c */
-extern int theHttpConnection;	/* main.c */
-extern int theInIcpConnection;	/* main.c */
-extern int theOutIcpConnection;	/* main.c */
-extern int vizSock;
-extern volatile int shutdown_pending;	/* main.c */
-extern volatile int reread_pending;	/* main.c */
-extern int opt_unlink_on_reload;	/* main.c */
-extern int opt_reload_hit_only;	/* main.c */
-extern int opt_dns_tests;	/* main.c */
-extern int opt_foreground_rebuild;	/* main.c */
-extern int opt_zap_disk_store;	/* main.c */
-extern int opt_syslog_enable;	/* main.c */
-extern int opt_catch_signals;	/* main.c */
-extern int opt_no_ipcache;	/* main.c */
-extern int vhost_mode;		/* main.c */
-extern int Squid_MaxFD;		/* main.c */
-extern int Biggest_FD;		/* main.c */
-extern int Number_FD;		/* main.c */
-extern int select_loops;	/* main.c */
-extern const char *const version_string;	/* main.c */
-extern const char *const appname;	/* main.c */
-extern struct in_addr local_addr;	/* main.c */
-extern struct in_addr theOutICPAddr;	/* main.c */
-extern const char *const localhost;
-extern struct in_addr no_addr;	/* comm.c */
-extern int opt_udp_hit_obj;	/* main.c */
-extern int opt_mem_pools;	/* main.c */
-extern int opt_forwarded_for;	/* main.c */
-extern int opt_accel_uses_host;	/* main.c */
-extern char ThisCache[];	/* main.c */
+/*
+ * Squid source files should not call these functions directly
+ */
+#ifndef malloc
+#define malloc +
+#endif
+#ifndef free
+#define free +
+#endif
+#ifndef calloc
+#define calloc +
+#endif
+#ifndef sprintf
+#define sprintf +
+#endif
+#ifndef strdup
+#define strdup +
+#endif
 
-/* Prototypes and definitions which don't really deserve a separate
- * include file */
+#if SQUID_SNMP
+extern struct snmp_mib_tree *Mib;
+#endif
 
-#define  CONNECT_PORT        443
-
-extern void start_announce _PARAMS((void *unused));
-extern int sslStart _PARAMS((int fd, const char *, request_t *, char *, int *sz));
-extern const char *storeToString _PARAMS((const StoreEntry *));
-extern int waisStart _PARAMS((int, const char *, method_t, char *, StoreEntry *));
-extern void storeDirClean _PARAMS((void *unused));
-extern int passStart _PARAMS((int fd,
-	const char *url,
-	request_t * request,
-	char *buf,
-	int buflen,
-	int *size_ptr));
-extern void identStart _PARAMS((int, icpStateData *,
-	void       (*callback) _PARAMS((void *))));
-extern int httpAnonAllowed _PARAMS((const char *line));
-extern int httpAnonDenied _PARAMS((const char *line));
-
-extern const char *const dash_str;
-extern const char *const null_string;
-
-#define OR(A,B) (A ? A : B)
 
 #endif /* SQUID_H */
