@@ -1,6 +1,6 @@
 
 /*
- * $Id: stat.c,v 1.297.2.2 1998/12/07 21:00:28 wessels Exp $
+ * $Id: stat.c,v 1.311.2.5 1999/09/23 05:55:34 wessels Exp $
  *
  * DEBUG: section 18    Cache Manager Statistics
  * AUTHOR: Harvest Derived
@@ -76,8 +76,7 @@ static OBJH statAvg5min;
 static OBJH statAvg60min;
 static OBJH statUtilization;
 static OBJH statCountersHistograms;
-static double statRequestHitRatio(int minutes);
-static double statByteHitRatio(int minutes);
+static OBJH statClientRequests;
 
 #ifdef XMALLOC_STATISTICS
 static void info_get_mallstat(int, int, StoreEntry *);
@@ -221,12 +220,16 @@ storeEntryFlags(const StoreEntry * entry)
 	strcat(buf, "DISPATCHED,");
     if (EBIT_TEST(flags, KEY_PRIVATE))
 	strcat(buf, "PRIVATE,");
+    if (EBIT_TEST(flags, ENTRY_FWD_HDR_WAIT))
+	strcat(buf, "FWD_HDR_WAIT,");
     if (EBIT_TEST(flags, ENTRY_NEGCACHED))
 	strcat(buf, "NEGCACHED,");
     if (EBIT_TEST(flags, ENTRY_VALIDATED))
 	strcat(buf, "VALIDATED,");
     if (EBIT_TEST(flags, ENTRY_BAD_LENGTH))
 	strcat(buf, "BAD_LENGTH,");
+    if (EBIT_TEST(flags, ENTRY_ABORTED))
+	strcat(buf, "ABORTED,");
     if ((t = strrchr(buf, ',')))
 	*t = '\0';
     return buf;
@@ -273,7 +276,7 @@ statStoreEntry(StoreEntry * s, StoreEntry * e)
 	for (i = 0, sc = &mem->clients[i]; sc != NULL; sc = sc->next, i++) {
 	    if (sc->callback_data == NULL)
 		continue;
-	    storeAppendPrintf(s, "\tClient #%d\n", i);
+	    storeAppendPrintf(s, "\tClient #%d, %p\n", i, sc->callback_data);
 	    storeAppendPrintf(s, "\t\tcopy_offset: %d\n",
 		(int) sc->copy_offset);
 	    storeAppendPrintf(s, "\t\tseen_offset: %d\n",
@@ -308,7 +311,7 @@ statObjects(void *data)
 	storeUnlockObject(state->sentry);
 	cbdataFree(state);
 	return;
-    } else if (state->sentry->store_status == STORE_ABORTED) {
+    } else if (EBIT_TEST(state->sentry->flags, ENTRY_ABORTED)) {
 	storeUnlockObject(state->sentry);
 	cbdataFree(state);
 	return;
@@ -338,7 +341,7 @@ statObjectsStart(StoreEntry * sentry, STOBJFLT * filter)
     state->sentry = sentry;
     state->filter = filter;
     storeLockObject(sentry);
-    cbdataAdd(state, MEM_NONE);
+    cbdataAdd(state, cbdataXfree, 0);
     eventAdd("statObjects", statObjects, state, 0.0, 1);
 }
 
@@ -404,7 +407,7 @@ statFiledescriptors(StoreEntry * sentry)
     int i;
     fde *f;
     storeAppendPrintf(sentry, "Active file descriptors:\n");
-    storeAppendPrintf(sentry, "%-4s %-6s %-4s %-7s %-7s %-21s %s\n",
+    storeAppendPrintf(sentry, "%-4s %-6s %-4s %-7s* %-7s* %-21s %s\n",
 	"File",
 	"Type",
 	"Tout",
@@ -412,17 +415,19 @@ statFiledescriptors(StoreEntry * sentry)
 	"Nwrite",
 	"Remote Address",
 	"Description");
-    storeAppendPrintf(sentry, "---- ------ ---- ------- ------- --------------------- ------------------------------\n");
+    storeAppendPrintf(sentry, "---- ------ ---- -------- -------- --------------------- ------------------------------\n");
     for (i = 0; i < Squid_MaxFD; i++) {
 	f = &fd_table[i];
-	if (!f->open)
+	if (!f->flags.open)
 	    continue;
-	storeAppendPrintf(sentry, "%4d %-6.6s %4d %7d %7d %-21s %s\n",
+	storeAppendPrintf(sentry, "%4d %-6.6s %4d %7d%c %7d%c %-21s %s\n",
 	    i,
 	    fdTypeStr[f->type],
 	    f->timeout_handler ? (int) (f->timeout - squid_curtime) / 60 : 0,
 	    f->bytes_read,
+	    f->read_handler ? '*' : ' ',
 	    f->bytes_written,
+	    f->write_handler ? '*' : ' ',
 	    fdRemoteAddr(f),
 	    f->desc);
     }
@@ -436,7 +441,7 @@ info_get(StoreEntry * sentry)
     double runtime;
 #if HAVE_MSTATS && HAVE_GNUMALLOC_H
     struct mstats ms;
-#elif HAVE_MALLINFO
+#elif HAVE_MALLINFO && HAVE_STRUCT_MALLINFO
     struct mallinfo mp;
     int t;
 #endif
@@ -538,7 +543,7 @@ info_get(StoreEntry * sentry)
 	ms.bytes_total >> 10);
     storeAppendPrintf(sentry, "\tTotal free:            %6d KB %d%%\n",
 	ms.bytes_free >> 10, percent(ms.bytes_free, ms.bytes_total));
-#elif HAVE_MALLINFO
+#elif HAVE_MALLINFO && HAVE_STRUCT_MALLINFO
     mp = mallinfo();
     storeAppendPrintf(sentry, "Memory usage for %s via mallinfo():\n",
 	appname);
@@ -584,12 +589,14 @@ info_get(StoreEntry * sentry)
 	Biggest_FD);
     storeAppendPrintf(sentry, "\tNumber of file desc currently in use: %4d\n",
 	Number_FD);
+    storeAppendPrintf(sentry, "\tFiles queued for open:                %4d\n",
+	Opening_FD);
     storeAppendPrintf(sentry, "\tAvailable number of file descriptors: %4d\n",
-	Squid_MaxFD - Number_FD);
+	fdNFree());
     storeAppendPrintf(sentry, "\tReserved number of file descriptors:  %4d\n",
 	RESERVED_FD);
-    storeAppendPrintf(sentry, "\tDisk files open:                      %4d\n",
-	open_disk_fd);
+    storeAppendPrintf(sentry, "\tStore Disk files open:                %4d\n",
+	store_open_disk_fd);
 
     storeAppendPrintf(sentry, "Internal Data Structures:\n");
     storeAppendPrintf(sentry, "\t%6d StoreEntries\n",
@@ -857,6 +864,11 @@ statInit(void)
     cachemgrRegister("histograms",
 	"Full Histogram Counts",
 	statCountersHistograms, 0, 1);
+    ClientActiveRequests.head = NULL;
+    ClientActiveRequests.tail = NULL;
+    cachemgrRegister("active_requests",
+	"Client-side Active Requests",
+	statClientRequests, 0, 1);
 }
 
 static void
@@ -866,7 +878,7 @@ statAvgTick(void *notused)
     StatCounters *p = &CountHist[1];
     StatCounters *c = &Counter;
     struct rusage rusage;
-    eventAdd("statAvgTick", statAvgTick, NULL, COUNT_INTERVAL, 1);
+    eventAdd("statAvgTick", statAvgTick, NULL, (double) COUNT_INTERVAL, 1);
     squid_getrusage(&rusage);
     c->page_faults = rusage_pagefaults(&rusage);
     c->cputime = rusage_cputime(&rusage);
@@ -1143,27 +1155,10 @@ statPeerSelect(StoreEntry * sentry)
     /* per-peer */
     storeAppendPrintf(sentry, "\nPer-peer statistics:\n");
     for (peer = getFirstPeer(); peer; peer = getNextPeer(peer)) {
-	cacheDigestGuessStatsReport(&peer->digest.stats.guess, sentry, peer->host);
-	storeAppendPrintf(sentry, "peer.msgs_sent = %d\n",
-	    peer->digest.stats.msgs_sent);
-	storeAppendPrintf(sentry, "peer.msgs_recv = %d\n",
-	    peer->digest.stats.msgs_recv);
-	storeAppendPrintf(sentry, "peer.kbytes_sent = %d\n",
-	    (int) peer->digest.stats.kbytes_sent.kb);
-	storeAppendPrintf(sentry, "peer.kbytes_recv = %d\n",
-	    (int) peer->digest.stats.kbytes_recv.kb);
-	storeAppendPrintf(sentry, "peer.local_memory = %d\n",
-	    peer->digest.cd ? peer->digest.cd->mask_size / 1024 : 0);
-	storeAppendPrintf(sentry, "digest state: inited: %d, disabled: %d usable: %d requested: %d\n",
-	    0 < peer->digest.flags.inited,
-	    0 < peer->digest.flags.disabled,
-	    0 < peer->digest.flags.usable,
-	    0 < peer->digest.flags.requested
-	    );
-	if (peer->digest.cd)
-	    cacheDigestReport(peer->digest.cd, peer->host, sentry);
+	if (peer->digest)
+	    peerDigestStatsReport(peer->digest, sentry);
 	else
-	    storeAppendPrintf(sentry, "no cache digest from peer %s\n", peer->host);
+	    storeAppendPrintf(sentry, "\nNo peer digest from %s\n", peer->host);
 	storeAppendPrintf(sentry, "\n");
     }
 
@@ -1278,7 +1273,7 @@ statCPUUsage(int minutes)
 	tvSubDsec(CountHist[minutes].timestamp, CountHist[0].timestamp));
 }
 
-static double
+extern double
 statRequestHitRatio(int minutes)
 {
     assert(minutes < N_COUNT_HIST);
@@ -1288,7 +1283,7 @@ statRequestHitRatio(int minutes)
 	CountHist[minutes].client_http.requests);
 }
 
-static double
+extern double
 statByteHitRatio(int minutes)
 {
     size_t s;
@@ -1303,6 +1298,54 @@ statByteHitRatio(int minutes)
 	return (-1.0 * dpercent(s - c, c));
 }
 
+static void
+statClientRequests(StoreEntry * s)
+{
+    dlink_node *i;
+    clientHttpRequest *http;
+    ConnStateData *conn;
+    StoreEntry *e;
+    int fd;
+    for (i = ClientActiveRequests.head; i; i = i->next) {
+	http = i->data;
+	assert(http);
+	conn = http->conn;
+	storeAppendPrintf(s, "Connection: %p\n", conn);
+	if (conn) {
+	    fd = conn->fd;
+	    storeAppendPrintf(s, "\tFD %d, read %d, wrote %d\n", fd,
+		fd_table[fd].bytes_read, fd_table[fd].bytes_written);
+	    storeAppendPrintf(s, "\tFD desc: %s\n", fd_table[fd].desc);
+	    storeAppendPrintf(s, "\tin: buf %p, offset %d, size %d\n",
+		conn->in.buf, conn->in.offset, conn->in.size);
+	    storeAppendPrintf(s, "\tpeer: %s:%d\n",
+		inet_ntoa(conn->peer.sin_addr),
+		ntohs(conn->peer.sin_port));
+	    storeAppendPrintf(s, "\tme: %s:%d\n",
+		inet_ntoa(conn->me.sin_addr),
+		ntohs(conn->me.sin_port));
+	    storeAppendPrintf(s, "\tnrequests: %d\n",
+		conn->nrequests);
+	    storeAppendPrintf(s, "\tpersistent: %d\n",
+		conn->persistent);
+	    storeAppendPrintf(s, "\tdefer: n %d, until %d\n",
+		conn->defer.n, conn->defer.until);
+	}
+	storeAppendPrintf(s, "uri %s\n", http->uri);
+	storeAppendPrintf(s, "out.offset %d, out.size %d\n",
+	    http->out.offset, http->out.size);
+	storeAppendPrintf(s, "req_sz %d\n", http->req_sz);
+	e = http->entry;
+	storeAppendPrintf(s, "entry %p/%s\n", e, e ? storeKeyText(e->key) : "N/A");
+	e = http->old_entry;
+	storeAppendPrintf(s, "old_entry %p/%s\n", e, e ? storeKeyText(e->key) : "N/A");
+	storeAppendPrintf(s, "start %d.%06d (%f seconds ago)\n", http->start.tv_sec,
+	    http->start.tv_usec,
+	    tvSubDsec(http->start, current_time));
+	storeAppendPrintf(s, "\n");
+    }
+}
+
 #if STAT_GRAPHS
 /*
  * urgh, i don't like these, but they do cut the amount of code down immensely
