@@ -1,6 +1,6 @@
 
 /*
- * $Id: store.c,v 1.469.2.7 1998/12/09 04:11:23 wessels Exp $
+ * $Id: store.c,v 1.488.2.2 1999/05/10 16:02:04 wessels Exp $
  *
  * DEBUG: section 20    Storage Manager
  * AUTHOR: Harvest Derived
@@ -49,15 +49,13 @@ const char *pingStatusStr[] =
 {
     "PING_NONE",
     "PING_WAITING",
-    "PING_TIMEOUT",
     "PING_DONE"
 };
 
 const char *storeStatusStr[] =
 {
     "STORE_OK",
-    "STORE_PENDING",
-    "STORE_ABORTED"
+    "STORE_PENDING"
 };
 
 const char *swapStatusStr[] =
@@ -95,8 +93,7 @@ static EVH storeLateRelease;
  * local variables
  */
 static dlink_list inmem_list;
-static int store_pages_high = 0;
-static int store_pages_low = 0;
+static int store_pages_max = 0;
 static int store_swap_high = 0;
 static int store_swap_low = 0;
 static int store_swap_mid = 0;
@@ -157,7 +154,7 @@ destroy_MemObject(StoreEntry * e)
     ctx_exit(ctx);		/* must exit before we free mem->url */
     safe_free(mem->url);
     safe_free(mem->log_url);
-    memFree(MEM_MEMOBJECT, mem);
+    memFree(mem, MEM_MEMOBJECT);
 }
 
 static void
@@ -170,7 +167,7 @@ destroy_StoreEntry(void *data)
 	destroy_MemObject(e);
     storeHashDelete(e);
     assert(e->key == NULL);
-    memFree(MEM_STOREENTRY, e);
+    memFree(e, MEM_STOREENTRY);
 }
 
 /* ----- INTERFACE BETWEEN STORAGE MANAGER AND HASH TABLE FUNCTIONS --------- */
@@ -250,10 +247,8 @@ storeUnlockObject(StoreEntry * e)
 	storeKeyText(e->key), e->lock_count);
     if (e->lock_count)
 	return (int) e->lock_count;
-    if (e->store_status == STORE_PENDING) {
-	assert(!EBIT_TEST(e->flags, ENTRY_DISPATCHED));
+    if (e->store_status == STORE_PENDING)
 	EBIT_SET(e->flags, RELEASE_REQUEST);
-    }
     assert(storePendingNClients(e) == 0);
     if (EBIT_TEST(e->flags, RELEASE_REQUEST))
 	storeRelease(e);
@@ -283,10 +278,7 @@ storeGet(const cache_key * key)
 StoreEntry *
 storeGetPublic(const char *uri, const method_t method)
 {
-    StoreEntry *e = storeGet(storeKeyPublic(uri, method));
-    if (e == NULL && squid_curtime < 920000000)
-	e = storeGet(storeKeyPublicOld(uri, method));
-    return e;
+    return storeGet(storeKeyPublic(uri, method));
 }
 
 static int
@@ -387,9 +379,6 @@ storeCreateEntry(const char *url, const char *log_url, request_flags flags, meth
     e->timestamp = 0;		/* set in storeTimestampsSet() */
     e->ping_status = PING_NONE;
     EBIT_SET(e->flags, ENTRY_VALIDATED);
-#ifdef PPNR_WIP
-    EBIT_SET(e->flags, ENTRY_FWD_HDR_WAIT);
-#endif /* PPNR_WIP */
     return e;
 }
 
@@ -408,6 +397,7 @@ storeAppend(StoreEntry * e, const char *buf, int len)
     MemObject *mem = e->mem_obj;
     assert(mem != NULL);
     assert(len >= 0);
+    assert(e->store_status == STORE_PENDING);
     if (len) {
 	debug(20, 5) ("storeAppend: appending %d bytes for '%s'\n",
 	    len,
@@ -428,17 +418,18 @@ storeAppend(StoreEntry * e, const char *buf, int len)
 #endif
 }
 
-#ifdef __STDC__
 void
+#if STDC_HEADERS
 storeAppendPrintf(StoreEntry * e, const char *fmt,...)
+#else
+storeAppendPrintf(va_alist)
+     va_dcl
+#endif
 {
+#if STDC_HEADERS
     va_list args;
     va_start(args, fmt);
 #else
-void
-storeAppendPrintf(va_alist)
-     va_dcl
-{
     va_list args;
     StoreEntry *e = NULL;
     const char *fmt = NULL;
@@ -470,6 +461,7 @@ struct _store_check_cachable_hist {
 	int too_big;
 	int private_key;
 	int too_many_open_files;
+	int too_many_open_fds;
 	int lru_age_too_low;
     } no;
     struct {
@@ -482,7 +474,7 @@ storeTooManyDiskFilesOpen(void)
 {
     if (Config.max_open_disk_fds == 0)
 	return 0;
-    if (open_disk_fd > Config.max_open_disk_fds)
+    if (store_open_disk_fd > Config.max_open_disk_fds)
 	return 1;
     return 0;
 }
@@ -515,9 +507,19 @@ storeCheckCachable(StoreEntry * e)
     } else if (EBIT_TEST(e->flags, KEY_PRIVATE)) {
 	debug(20, 3) ("storeCheckCachable: NO: private key\n");
 	store_check_cachable_hist.no.private_key++;
+    } else if (e->swap_status != SWAPOUT_NONE) {
+	/*
+	 * here we checked the swap_status because the remaining
+	 * cases are only relevant only if we haven't started swapping
+	 * out the object yet.
+	 */
+	return 1;
     } else if (storeTooManyDiskFilesOpen()) {
 	debug(20, 2) ("storeCheckCachable: NO: too many disk files open\n");
 	store_check_cachable_hist.no.too_many_open_files++;
+    } else if (fdNFree() < RESERVED_FD) {
+	debug(20, 2) ("storeCheckCachable: NO: too many FD's open\n");
+	store_check_cachable_hist.no.too_many_open_fds++;
     } else if (storeExpiredReferenceAge() < 300) {
 	debug(20, 2) ("storeCheckCachable: NO: LRU Age = %d\n",
 	    storeExpiredReferenceAge());
@@ -534,6 +536,8 @@ storeCheckCachable(StoreEntry * e)
 static void
 storeCheckCachableStats(StoreEntry * sentry)
 {
+    storeAppendPrintf(sentry, "Category\t Count\n");
+
     storeAppendPrintf(sentry, "no.non_get\t%d\n",
 	store_check_cachable_hist.no.non_get);
     storeAppendPrintf(sentry, "no.not_entry_cachable\t%d\n",
@@ -550,6 +554,8 @@ storeCheckCachableStats(StoreEntry * sentry)
 	store_check_cachable_hist.no.private_key);
     storeAppendPrintf(sentry, "no.too_many_open_files\t%d\n",
 	store_check_cachable_hist.no.too_many_open_files);
+    storeAppendPrintf(sentry, "no.too_many_open_fds\t%d\n",
+	store_check_cachable_hist.no.too_many_open_fds);
     storeAppendPrintf(sentry, "no.lru_age_too_low\t%d\n",
 	store_check_cachable_hist.no.lru_age_too_low);
     storeAppendPrintf(sentry, "yes.default\t%d\n",
@@ -561,7 +567,15 @@ void
 storeComplete(StoreEntry * e)
 {
     debug(20, 3) ("storeComplete: '%s'\n", storeKeyText(e->key));
-    assert(e->store_status == STORE_PENDING);
+    if (e->store_status != STORE_PENDING) {
+	/*
+	 * if we're not STORE_PENDING, then probably we got aborted
+	 * and there should be NO clients on this entry
+	 */
+	assert(EBIT_TEST(e->flags, ENTRY_ABORTED));
+	assert(e->mem_obj->nclients == 0);
+	return;
+    }
     e->mem_obj->object_sz = e->mem_obj->inmem_hi;
     e->store_status = STORE_OK;
     assert(e->mem_status == NOT_IN_MEMORY);
@@ -577,49 +591,40 @@ storeComplete(StoreEntry * e)
     storeCheckSwapOut(e);
 }
 
-#ifdef PPNR_WIP
-void
-storePPNR(StoreEntry * e)
-{
-    assert(EBIT_TEST(e->flags, ENTRY_FWD_HDR_WAIT));
-    EBIT_CLR(e->flags, ENTRY_FWD_HDR_WAIT);
-}
-
-#endif /* PPNR_WIP */
-
 /*
  * Someone wants to abort this transfer.  Set the reason in the
  * request structure, call the server-side callback and mark the
  * entry for releasing 
  */
 void
-storeAbort(StoreEntry * e, int cbflag)
+storeAbort(StoreEntry * e)
 {
     MemObject *mem = e->mem_obj;
-    STABH *callback;
-    void *data;
     assert(e->store_status == STORE_PENDING);
     assert(mem != NULL);
     debug(20, 6) ("storeAbort: %s\n", storeKeyText(e->key));
     storeLockObject(e);		/* lock while aborting */
     storeNegativeCache(e);
     storeReleaseRequest(e);
-    e->store_status = STORE_ABORTED;
+    EBIT_SET(e->flags, ENTRY_ABORTED);
     storeSetMemStatus(e, NOT_IN_MEMORY);
     /* No DISK swap for negative cached object */
     e->swap_status = SWAPOUT_NONE;
+    e->store_status = STORE_OK;
     /*
      * We assign an object length here.  The only other place we assign
      * the object length is in storeComplete()
      */
     mem->object_sz = mem->inmem_hi;
     /* Notify the server side */
-    if (cbflag && mem->abort.callback) {
-	callback = mem->abort.callback;
-	data = mem->abort.data;
+    if (mem->abort.callback) {
+	eventAdd("mem->abort.callback",
+	    mem->abort.callback,
+	    mem->abort.data,
+	    0.0,
+	    0);
 	mem->abort.callback = NULL;
 	mem->abort.data = NULL;
-	callback(data);
     }
     /* Notify the client side */
     InvokeHandlers(e);
@@ -657,7 +662,7 @@ storeGetMemSpace(int size)
 	return;
     last_check = squid_curtime;
     pages_needed = (size / SM_PAGE_SIZE) + 1;
-    if (memInUse(MEM_STMEM_BUF) + pages_needed < store_pages_high)
+    if (memInUse(MEM_STMEM_BUF) + pages_needed < store_pages_max)
 	return;
     if (store_rebuilding)
 	return;
@@ -675,7 +680,7 @@ storeGetMemSpace(int size)
 	}
 	released++;
 	storePurgeMem(e);
-	if (memInUse(MEM_STMEM_BUF) + pages_needed < store_pages_high)
+	if (memInUse(MEM_STMEM_BUF) + pages_needed < store_pages_max)
 	    break;
     }
     debug(20, 3) ("storeGetMemSpace stats:\n");
@@ -705,25 +710,21 @@ storeMaintainSwapSpace(void *datanotused)
     int expired = 0;
     int max_scan;
     int max_remove;
+    double f;
     static time_t last_warn_time = 0;
     /* We can't delete objects while rebuilding swap */
     if (store_rebuilding) {
 	eventAdd("MaintainSwapSpace", storeMaintainSwapSpace, NULL, 1.0, 1);
 	return;
-    } else if (store_swap_size < store_swap_mid) {
-	max_scan = 100;
-	max_remove = 8;
-	eventAdd("MaintainSwapSpace", storeMaintainSwapSpace, NULL, 1.0, 1);
-    } else if (store_swap_size < store_swap_high) {
-	max_scan = 200;
-	max_remove = 8;
-	eventAdd("MaintainSwapSpace", storeMaintainSwapSpace, NULL, 0.1, 1);
     } else {
-	max_scan = 500;
-	max_remove = 32;
-	eventAdd("MaintainSwapSpace", storeMaintainSwapSpace, NULL, 0.0, 1);
+	f = (double) (store_swap_size - store_swap_low) / (store_swap_high - store_swap_low);
+	f = f < 0.0 ? 0.0 : f > 1.0 ? 1.0 : f;
+	max_scan = (int) (f * 400.0 + 100.0);
+	max_remove = (int) (f * 70.0 + 10.0);
+	eventAdd("MaintainSwapSpace", storeMaintainSwapSpace, NULL, 1.0 - f, 1);
     }
-    debug(20, 3) ("storeMaintainSwapSpace\n");
+    debug(20, 3) ("storeMaintainSwapSpace: f=%f, max_scan=%d, max_remove=%d\n",
+	f, max_scan, max_remove);
     for (m = store_list.tail; m; m = prev) {
 	prev = m->prev;
 	e = m->data;
@@ -919,7 +920,7 @@ storeInitHashValues(void)
 	store_hash_buckets,
 	store_maintain_rate,
 	store_maintain_rate == 1 ? null_string : "s");
-    debug(20, 1) ("Max Mem  size: %d KB\n", Config.Mem.maxSize >> 10);
+    debug(20, 1) ("Max Mem  size: %d KB\n", Config.memMaxSize >> 10);
     debug(20, 1) ("Max Swap size: %d KB\n", Config.Swap.maxSize);
 }
 
@@ -957,21 +958,12 @@ storeInit(void)
 void
 storeConfigure(void)
 {
-    int store_mem_high = 0;
-    int store_mem_low = 0;
-    store_mem_high = (long) (Config.Mem.maxSize / 100) *
-	Config.Mem.highWaterMark;
-    store_mem_low = (long) (Config.Mem.maxSize / 100) *
-	Config.Mem.lowWaterMark;
-
     store_swap_high = (long) (((float) Config.Swap.maxSize *
 	    (float) Config.Swap.highWaterMark) / (float) 100);
     store_swap_low = (long) (((float) Config.Swap.maxSize *
 	    (float) Config.Swap.lowWaterMark) / (float) 100);
     store_swap_mid = (store_swap_high >> 1) + (store_swap_low >> 1);
-
-    store_pages_high = store_mem_high / SM_PAGE_SIZE;
-    store_pages_low = store_mem_low / SM_PAGE_SIZE;
+    store_pages_max = Config.memMaxSize / SM_PAGE_SIZE;
 }
 
 static int
@@ -1063,7 +1055,7 @@ storeEntryValidToSend(StoreEntry * e)
     if (EBIT_TEST(e->flags, ENTRY_NEGCACHED))
 	if (e->expires <= squid_curtime)
 	    return 0;
-    if (e->store_status == STORE_ABORTED)
+    if (EBIT_TEST(e->flags, ENTRY_ABORTED))
 	return 0;
     return 1;
 }
@@ -1240,3 +1232,15 @@ storeEntryReply(StoreEntry * e)
 	return NULL;
     return e->mem_obj->reply;
 }
+
+void
+storeEntryReset(StoreEntry * e)
+{
+    MemObject *mem = e->mem_obj;
+    debug(20, 3) ("storeEntryReset: %s\n", storeUrl(e));
+    assert(mem->swapout.fd == -1);
+    stmemFree(&mem->data_hdr);
+    mem->inmem_hi = mem->inmem_lo = 0;
+    httpReplyDestroy(mem->reply);
+    mem->reply = httpReplyCreate();
+}
